<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: JsbuildController.php 3439 2013-06-12 13:15:07Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class JsbuildController extends Zend_Controller_Action
{

    protected $_default = array(
        'Util/Namespace.js' ,
        'Util.js' ,
        'Util/Translate.js' ,
        'Init.js' ,
        'Log.js' ,
    );
    
    protected $_map = array(
        'Util/Namespace.js' ,
        'Util.js' , 
        'Util/Translate.js' ,
        'Util/Date.js' ,
        'Init.js' ,
        'Util/Photobox.js' ,
        'Util/Tooltip.js' ,
        'Log.js' ,
        'Map.js' ,
        'Control.js' ,
        'Color/Picker.js' ,
        //'Calendar/Interval.js' ,
        'Datatable.js' ,
        'Treeview.js',
        'Control/Click.js' ,
        'Control/Scale.js' ,
        'Control/Publiclayers.js' ,
        'Control/Privatelayers.js' ,
        'Control/Legend.js' ,
        'Control/Geocode.js' ,
    
        'Control/Zoomregion.js' ,
        'Control/Fullextent.js' ,
        'Control/Info.js' ,
        'Control/Search.js' ,
        'Control/Measure.js' ,
        'Control/Geolocation.js' ,
        'Control/Querybuilder.js' ,
        'Control/Rastercut.js' ,
        'Control/AdminData.js',
        array(
            'path' => 'Control/Print.js',
            'order' => 99
        ) ,
    );

    protected $_admin = array(
        'Util/Namespace.js' ,
        'Util.js' , 
        'Util/Translate.js' ,
        'Util/Date.js' ,
        'Log.js' ,
        'Color/Picker.js' ,
        'Color/Celleditor.js' ,
        'Datatable.js' ,
        'Treeview.js' ,
    );
    
    protected $_mobile = array(
        'Util/Namespace.js' ,
        'Util.js' , 
        'Util/Translate.js' ,
    );
    
    protected $_mobilemap = array(
        'Mobile/Jqm/Form.js' ,
        'Mobile/Jqm/ColorPicker.js' ,
        'Util/Namespace.js' ,
        'Util.js' , 
        'Util/Translate.js' ,
        'Util/Date.js' ,
        'Log.js' ,
        'Map.js' ,
        'Mobile/Map.js' ,
        'Control.js' ,
    	'Util/MobilePhotobox.js' ,
    
        'Mobile/Control/Offline.js' ,
        'Control/Click.js' ,
        'Control/Measure.js' ,
        'Control/Geolocation.js' ,
        //'Control/LoadingPanel.js' ,
        'Mobile/Control/Privatelayers.js' ,
        'Mobile/Control/Publiclayers.js' ,
        'Mobile/Control/Info.js' ,
        'Mobile/Control/Search.js' ,
        'Mobile/Control/Measure.js' ,
        'Mobile/Control/Geolocation.js' ,
        'Mobile/Control/Geocode.js',
        'Mobile/Control/Legend.js' ,
        array(
            'path' => 'Mobile/Control/Home.js',
            'order' => 99
        ) ,
    );
    
    public function indexAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $request = $this->getRequest();
        $type = $request->getParam('type');
        
        if(empty($type)) {
            $type = 'default';
        }
                
        if (Zend_Registry::isRegistered('cache')) {
            $cache = Zend_Registry::get('cache');
            $cacheId = md5(GT_APP_PATH) . '_jsbuild_' . $type;
            
            if (! $cache->test($cacheId)) {
                $js = $this->getJs($type);
                $cache->save($js, $cacheId);
            } else {
                $js = $cache->load($cacheId);
            }
            
        } else {
            $js = $this->getJs($type);
        }
        
        //Cordova
        $matches = NULL;
        if(isset($_SERVER['HTTP_USER_AGENT'])) {
            $useragent = $_SERVER['HTTP_USER_AGENT'];
        } else {
            $useragent = '';
        }
        if(preg_match('/Cordova (\d\.\d\.\d)/', $useragent, $matches)) {
            $cordovaFilePath = GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'cordova-' . $matches[1] . '.js';
            if(file_exists($cordovaFilePath)) {
                $js .= file_get_contents($cordovaFilePath) . "\n";
            }
        }
        
        header("Content-type: application/javascript");
        
        //Expires
        if(GT_PRODUCTION == true) {
            $this->getResponse()->setHeader('Cache-Control', 'private, max-age=86400, pre-check=86400', true);
            $this->getResponse()->setHeader('Pragma', 'private', true);
        } else {
            /*
            $this->getResponse()->setHeader('Expires', 'Sat, 26 Jul 1997 05:00:00 GMT', true);
            $this->getResponse()->setHeader('Cache-Control', '', true);
            */
        }
            
        //Flush
        echo $js;
    }
    
    public function getJs($type)
    {
        $config = Zend_Registry::get('config');
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        if ($type == 'map' || $type == 'mobilemap') {
            //Add plugin javascript files
            foreach ($this->getFrontController()->getPlugins() as $plugin) {
                if (! is_subclass_of($plugin, 'Giscuit_Controller_Plugin_Abstract')) {
                    continue;
                }
                                
                if($type == 'map' && $plugin->getJsFilesList()) {
                    $this->_map = array_merge($this->_map, $plugin->getJsFilesList());
                }
                
                if($type == 'mobilemap' && $plugin->getMobileJsFilesList()) {
                    $this->_mobilemap = array_merge($this->_mobilemap, $plugin->getMobileJsFilesList());
                }
            }
            
            //Add layer javascript files
            $layersTable = new Giscuit_Db_Table_Layers();
            $mapOptions = array(
                'addEditLayer' => true , 
                'orderDirection' => 'asc' , 
                'whereActive' => 't' , 
            );
            $rows = $layersTable->getItems($mapOptions);

            foreach($rows as $layer) {
                if(! empty($layer['js_file'])) {
                    if($type == 'map') {
                        $this->_map[] = $layer['js_file'];
                    } else if($type == 'mobilemap') {
                        $this->_mobilemap[] = 'Mobile' . DIR_SEP . $layer['js_file'];
                    }               
                }
            }
        }
        
        $propName = '_' . $type;
        if(isset($this->$propName)) {
            $files = $this->$propName;
        } else {
            throw new Giscuit_Exception("Invalid type ({$type})");
        }
        
        $js = '';
        
        //GpsGate
        if (($type == 'map') || $type == 'mobilemap') {
            $js .= file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'GpsGate.js') . "\n";
        }

        //
        $order = array();
        foreach ($files as $key => $value) {
            if(is_string($value)) {
                $files[$key] = array(
                    'path' => $value,
                    'order' => $key
                );
            }
            $order[$key] = $files[$key]['order'];
        }
        array_multisort($order, SORT_NUMERIC, SORT_ASC, $files);
        
        
        $uniqueFiles = array();
        foreach ($files as $value) {
            if(in_array($value['path'], $uniqueFiles)) {
                continue;
            } 
            
            $uniqueFiles[] = $value['path'];
            
            $filePath = GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'giscuit' . DIR_SEP . $value['path'];
            if(file_exists($filePath)) {
                $js .= file_get_contents($filePath) . "\n";
            }
        }
        
        //Minify
        $js = $this->minify($js);
        
        //Accordion
        if($type == 'default' || $type == 'map') {
            $js .= file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'accordionview-min.js') . "\n";
        }
        
        //Add proj4js defition of the current projection
        if ($type == 'map' || $type == 'mobilemap') {
            //Add proj4js
            $js .= file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'proj4js-compressed.js') . "\n";
            
            if($config->map->srid != 4326 && $config->map->srid != 900913) {
                $geometryTablesTable = new Giscuit_Db_Table_GeometryTables();
                $projText = trim($geometryTablesTable->getProjTextBySrid($config->map->srid)->proj4text);
                $js .= "\nProj4js.defs['EPSG:{$config->map->srid}'] = '$projText';";
            }
        }
        
        return $js;
    }
    
    public static function minify($js) {
        $js = explode("\n", $js);
        $jsLen = count($js);
        $inMultiLineComment = false;
        
        for($i = 0; $i < $jsLen; $i++) {
            $js[$i] = trim($js[$i]);
            
            //
            if($js[$i] === '') {
                unset($js[$i]);
                continue;
            } 
            
            //
            if((strlen($js[$i]) > 1 && $js[$i][0] == '/' && $js[$i][1] == '/')) {
                unset($js[$i]);
                continue;
            }
            
            //
            if($js[$i] == '/**') {
                $inMultiLineComment = true;
                
                unset($js[$i]);
                continue;
            }
            if($js[$i] == '*/' && $inMultiLineComment) {
                $inMultiLineComment = false;
                
                unset($js[$i]);
                continue;
            }
            if($inMultiLineComment) {
                unset($js[$i]);
                continue;
            }
        }
        $js = implode("\n", $js);
        
        return $js;
    }

}