<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: HistorymanageController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_HistorymanageController extends Zend_Controller_Action
{
    
    public function indexAction ()
    {
        $settings = Zend_Registry::get('settings');
        $form = $this->getForm();
        
        $data = array();
        if (! empty($settings->historyDateBegin)) {
            $data['dateBegin'] = $settings->historyDateBegin;
        }
        if (! empty($settings->historyDateEnd)) {
            $data['dateEnd'] = $settings->historyDateEnd;
        }
        $form->populate($data);
        
        //Get a list of editable layers
        $this->view->layers = array();
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        foreach ($tables as $table) {
            $settings = $table->getSettings();
            $this->view->layers[] = array(
                'id' => $settings['id'] ,
                'name' => $settings['name']
            );
        }
                
        $this->view->form = $form;
    }

    public function submitAction ()
    {
        //Disable editing
        Zend_Registry::get('settings')->editingActive = false;
        
        $settings = Zend_Registry::get('settings');
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        
        $form = $this->getForm();
        $form->populate($request->getParams());
        
        if (! $form->isValid($request->getParams())) {

        } else {
            $settings->historyDateBegin = $form->getValue('dateBegin');
            $settings->historyDateEnd = $form->getValue('dateEnd');
        }
        
        $this->_helper->json($result);
    }
    
    public function getAction ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id', 0));
        
        $result = $fields = array();
        $layerFound = false;
        
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        
        foreach ($tables as $table) {
            $settings = $table->getSettings();
            if($settings['id'] == $id) {
                $layerFound = true;
                break;
            }
        }

        if($layerFound == false) {
            throw new Giscuit_Exception("Invalid layer id");
        }
        
        $historyName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $history = new $historyName(array(
            'name' => $settings['table'] , 
            'withHistory' => true
        ));
        $history->setSettings($settings);
        
        $select = $history->getSelect('all', array(
            'geometryAsWkt' => true , 
            'whereRawHistory' => true ,
            'fields' => array(
                'hstatus'
            )
        ));
        $select->order($history->info('name') . '.hgid desc');
        $select->limit(50);
        
        $rows = $history->fetchAll($select)->toArray();
        
        //Get fields
        foreach ($history->getPublicFields() as $field) {
            $fields[] = $field['name'];
        }
              
        //Column definitions
        $columnDefs = array(
            array(
                'key' => 'hstatus' , 
                'label' => $translate->_('hstatus') , 
                'sortable' => true , 
                'formatter' => "formatStatusField" , 
                'resizeable' => true
            )
        );
        foreach ($fields as $field) {
            $arrObj = new ArrayObject(array(
                'key' => $field , 
                'label' => $translate->_($field) , 
                'sortable' => true , 
                'resizeable' => true
            ));
            
            $columnDefs[] = $arrObj;
        }
        
        $result = array(
            'replyCode' => 200 ,
            'records' => $rows , 
            'fields' => array_merge($fields, array(
                'hgid' ,
                'the_geom' , 
                'hstatus'
            )) , 
            'name' => $settings['name'] , 
            'columns' => $columnDefs 
        );
        
        $this->_helper->json($result);

    }
    
    public function revertAction ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $layerId = intval($request->getParam('layer_id', 0));
        $hgid = intval($request->getParam('hgid', 0));
        
        $layerFound = false;
        $hstatus = null;
        
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        
        foreach ($tables as $table) {
            $settings = $table->getSettings();
            if($settings['id'] == $layerId) {
                $layerFound = true;
                break;
            }
        }

        if($layerFound == false) {
            throw new Giscuit_Exception("Invalid layer id");
        }
        
        //Table
        $tableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $table = new $tableName(array(
            'name' => $settings['table'] , 
            'withHistory' => false
        ));
        $table->setSettings($settings);
        $fields = $table->info('cols');
        
        //History table
        $historyName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $history = new $historyName(array(
            'name' => $settings['table'] , 
            'withHistory' => true
        ));
        $history->setSettings($settings);
        
        //Create form
        $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $settings['class'] . '.xml';
        $configContent = new Zend_Config_Xml($configPath);
        $formName = 'Giscuit_Form_Layer_' . $settings['class'];
        $form = new $formName($configContent);

        //Find history record
        $historyRows = $history->find($hgid);
        
        if(count($historyRows) == 0) {
            throw new Giscuit_Exception("Invalid hgid");
        }
        
        $historyRow = $historyRows->current();
        
        //Find record
        $rows = $table->find($historyRow->gid);
        if(count($rows)) {
            //Update
            
            $row = $rows->current();
            $hstatus = 'U';
        } else {
            //Create
            
            $row = $table->createRow();
            $hstatus = 'C';
        }
        
        //Update table
        foreach($fields as $field) {
            $row->$field = $historyRow->$field;
        }
                
        $row->save();
        
        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        $editingPlugin->setItemHistory($historyRow->gid, $table, array(
            'hstatus' => $hstatus
        ));
        
        $this->_helper->json($result);
    }

    public function getForm ()
    {
        $form = new Giscuit_Form_History();
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtHistorymanageForm'
        ));
        
        return $form;
    }
}

