<?php

class Tool_JobsController extends Zend_Controller_Action
{

    public function getAction ()
    {
        $result['replyCode'] = 200;

        $config = Zend_Registry::get('config');
        $request = $this->getRequest();
        
        //$completed = ($request->getParam('completed') == 'true') ? true : false;
        $limit = intval($request->getParam('limit'));
        $offset = intval($request->getParam('offset'));
        $sortCol = $request->getParam('sort');
        $sortDir = ($request->getParam('dir') == 'asc') ? 'asc' : 'desc';
        
        $jobsTable = new Giscuit_Db_Table_Jobs();
        
        $jobsColumns = $jobsTable->info('cols');
        $columnValidator = new Zend_Validate_InArray($jobsColumns);
        if (! $columnValidator->isValid($sortCol)) {
            $sortCol = 'date';
        }
        
        //Populate database options
        $jobParams = array();
        if ($limit) {
            $jobParams['limit'] = $limit;
            if ($offset) {
                $jobParams['offset'] = $offset;
            } else {
                $jobParams['offset'] = 0;
            }
        }
        
        if ($sortCol && $sortDir) {
            if ($sortCol == 'status' || $sortCol == 'priority') {
                $jobParams['order'] = array(
                    $sortCol . ' ' . $sortDir,
                    'date DESC'
                );
            } else {
                $jobParams['order'] = $sortCol . ' ' . $sortDir;
            }
        }
        
        $jobs = $jobsTable->getItems($jobParams);

        //Get total jobs
        $result['totalRecords'] = $jobsTable->getTotal($jobParams)->total;
        $result['offset'] = $offset;

        $result['records'] = $jobs;

        $this->_helper->json($result);
    }
    
    public function addAction() 
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $config = Zend_Registry::get('config');
        $form = $this->getForm('add');
        
        if ($request->isPost()) {
            $form->populate($request->getParams());
            if ($form->isValid($request->getParams())) {
                //Parse geometry
                $parsedGeometry = explode(' ', substr(substr($request->getParam('geometry'), 6), 0, -1));
                $geometry['x'] = floatval($parsedGeometry[0]);
                $geometry['y'] = floatval($parsedGeometry[1]);
                
                $jobsTable = new Giscuit_Db_Table_Jobs();
                $newJobId = $jobsTable->addItem($form, $geometry);
                
                if ($config->smtp->host && $config->smtp->port) {
                    $usersTable = new Giscuit_Db_Table_Users();
                    $userRecord = $usersTable->getItems(array('whereId' => $form->getValue('to_user')))->current();
                    if (! empty($userRecord->email) && $userRecord->email_active == 't') {
                        $this->sendAssignMail(array(
                            'id' => $newJobId,
                            'title' => $form->getValue('title'),
                            'desc' => $form->getValue('desc'),
                            'priority' => $form->getValue('priority'),
                            'email' => $userRecord->email
                        ), 'add');
                    }
                }
            } else {
                $this->view->form = $form;
                $result['form'] = $this->view->render('jobs' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
            }
        } else {
            $this->view->form = $form;
            $result['form'] = $this->view->render('jobs' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        if(GT_TESTING == false) {
            //FIXME: Avoid converting to html entities by the browser
            header("Content-Type: text/plain");
        }
        
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
    }
    
    public function editAction() 
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $jobsTable = new Giscuit_Db_Table_Jobs();
        $config = Zend_Registry::get('config');
        $id = intval($request->getParam('id'));
        $form = $this->getForm('edit');
        
        if ($request->isPost()) {
            $form->populate($request->getParams());
            if ($form->isValid($request->getParams())) {
                $jobsTable->editItem($id, $form);
                
                if ($config->smtp->host && $config->smtp->port) {
                    $usersTable = new Giscuit_Db_Table_Users();
                    $userRecord = $usersTable->getItems(array('whereId' => $form->getValue('to_user')))->current();
                    $this->sendAssignMail(array(
                        'id' => $id,
                        'title' => $form->getValue('title'),
                        'desc' => $form->getValue('desc'),
                        'priority' => $form->getValue('priority'),
                        'email' => $userRecord->email
                    ), 'edit');
                }
            } else {
                $this->view->form = $form;
                $result['form'] = $this->view->render('jobs' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
            }
        } else {
            $data = $jobsTable->getItems(array(
                'whereId' => $id  
            ));
            
            //HACK: Parse epoch into timestamp
            $deadline = new Zend_Date($data[0]['deadline'], Zend_Date::TIMESTAMP);
            $data[0]['deadline'] = $deadline->get(GT_DATETIME_STYLE_PHP);
                        
            $form->populate($data[0]);
            
            $this->view->form = $form;
            $result['form'] = $this->view->render('jobs' . DIR_SEP . 'getform' . GT_SCRIPT_EXT);
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        if(GT_TESTING == false) {
            //FIXME: Avoid converting to html entities by the browser
            header("Content-Type: text/plain");
        }
        
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
    }
    
    public function getdetailsAction() 
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $id = intval($request->getParam('id'));
        
        $jobsTable = new Giscuit_Db_Table_Jobs();
        $job = $jobsTable->getItems(array(
            'whereId' => $id,
            'withActions' => true,
            'withActionsWorkflows' => true
        ));
        
        $result['job'] = $job[0];
        
        //Set empty text if workflow was deleted
        if (empty($result['job']['name'])) {
            $result['job']['name'] = '';
        }
        
        $result['actions'] = $this->getWorkflowActions($job);
        
        $this->_helper->json($result);
    }
    
    public function deleteAction() 
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $id = intval($request->getParam('id'));
        
        $jobsTable = new Giscuit_Db_Table_Jobs();
        $where = $jobsTable->getAdapter()->quoteInto('id = ?', $id);
        $jobsTable->delete($where);
        
        $jobActionsTable = new Giscuit_Db_Table_JobsActions();
        $where = $jobActionsTable->getAdapter()->quoteInto('job_id = ?', $id);
        $jobActionsTable->delete($where);
        
        $this->_helper->json($result);
    }

    public function getworkflowsAction()
    {
        $result['replyCode'] = 200;
        
        $workflowsTable = new Giscuit_Db_Table_JobsActionsWorkflows();
        $result['records'] = $workflowsTable->getItems(array('withMultilangFields' => true))->toArray();

        $this->_helper->json($result);
    }
    
    public function addworkflowAction()
    {
        $result['replyCode'] = 200;

        $workflowsTable = new Giscuit_Db_Table_JobsActionsWorkflows();
        
        $result['record'] = $workflowsTable->addItem();
        
        $this->_helper->json($result);
    }
    
    public function updateworkflowAction()
    {
        $result['replyCode'] = 200;
        
        $textTable = new Giscuit_Db_Table_Text();
        $cols = $textTable->info('cols');
        foreach ($cols as $key => $value) {
            $cols[$key] = 'name_' . $value;
        }
        
        $filters = array(
            '*' => 'StringTrim'
        );
        
        $validators = array(
            'column' => new Zend_Validate_InArray($cols) , 
            'data' => array(
                'allowEmpty' => true
            ) , 
            'id' => 'Digits'
        );
        
        $input = new Zend_Filter_Input($filters, $validators, $this->getRequest()->getParams());
        
        if ($input->isValid()) {
            $rows = $textTable->find($input->id);
                
            if ($rows->count() == 0) {
                throw new Giscuit_Exception("Invalid text id :" . $input->id);
            }
                
            $row = $rows->current();
            $column = explode('_', $input->column);
            $column = $column[1];
            $row->$column = $input->data;
            $row->save();
        } else {
            $result['replyCode'] = 500;
            $result['errors'] = $input->getMessages();
        }
        
        $this->_helper->json($result);
    }
    
    public function deleteworkflowAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('ids'));

        //Delete workflow
        $workflowsTable = new Giscuit_Db_Table_JobsActionsWorkflows();
        $where = $workflowsTable->getAdapter()->quoteInto('id = ?', $id);
        $workflowsTable->delete($where);
        /*
        //Delete workflow actions
        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        $where = $actionsMapTable->getAdapter()->quoteInto('workflow_id = ?', $id);
        $actionsMapTable->delete($where);
        */
        $this->_helper->json($result);
    }
    
    public function getactionsAction()
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $id = intval($request->getParam('id'));
        
        if ($id) {
            $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
            $result['records'] = $actionsMapTable->getItems(array(
                'withMultilangFields' => true,
                'whereWorkflow' => array($id),
                'order' => 'order ASC'
            ))->toArray();
        } else {
            $result['records'] = array();
        }

        $this->_helper->json($result);
    }
    
    public function addactionAction()
    {
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        $workflowId = intval($request->getParam('workflowId'));

        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        
        $result['record'] = $actionsMapTable->addItem($workflowId);
        
        $this->_helper->json($result);
    }
    
    public function updateactionAction()
    {
        $result['replyCode'] = 200;
        
        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        $textTable = new Giscuit_Db_Table_Text();
        
        $textCols = $textTable->info('cols');
        foreach ($textCols as $key => $value) {
            $textCols[$key] = 'name_' . $value;
        }
        $cols = array_merge($actionsMapTable->info('cols'), $textCols);
        
        $filters = array(
            '*' => 'StringTrim'
        );
        
        $validators = array(
            'column' => new Zend_Validate_InArray($cols) , 
            'data' => array(
                'allowEmpty' => true
            ) , 
            'id' => 'Digits'
        );
        
        $input = new Zend_Filter_Input($filters, $validators, $this->getRequest()->getParams());
        
        if ($input->isValid()) {
            if ($input->column == 'theme') {
                $themesArr = array('a', 'b', 'c', 'd', 'e');
                $themesValidator = new Zend_Validate_InArray($themesArr);
                
                if (! $themesValidator->isValid($input->data)) {
                    $input->data = 'c';
                }
                $actionsMapTable->editItem($input->id, $input);
            } else if ($input->column == 'parent') {
                $parentIds = explode(',', $input->data);
                
                $parsedIds = array();
                foreach ($parentIds as $id) {
                    $parsedIds[] = intval($id);
                }
                $input->data = implode(',', $parsedIds);
                
                $actionsMapTable->editItem($input->id, $input);
            } else {
                $rows = $textTable->find($input->id);
                
                if ($rows->count() == 0) {
                    throw new Giscuit_Exception("Invalid text id :" . $input->id);
                }
                
                $row = $rows->current();
                $column = explode('_', $input->column);
                $column = $column[1];
                $row->$column = $input->data;
                $row->save();
            }
        } else {
            $result['replyCode'] = 500;
            $result['errors'] = $input->getMessages();
        }
        
        $this->_helper->json($result);
    }
    
    public function deleteactionAction()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('ids'));

        $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        $where = $actionsMapTable->getAdapter()->quoteInto('id = ?', $id);
        $actionsMapTable->delete($where);
        
        $this->_helper->json($result);
    }
    
    public function moveactionAction ()
    {
        $request = $this->getRequest();
        
        //Input
        $first = intval($request->getParam('first'));
        $second = intval($request->getParam('second'));
        
        if (empty($first) || empty($second)) {
            $result['replyCode'] = 500;
            $result['replyError'] = 'Invalid input';
        } else {
            $result['replyCode'] = 200;
            
            $actionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
            $actionsMapTable->moveItem($first, $second);
        }
        
        $this->_helper->json($result);
    }
    
    public function sendAssignMail($data, $action)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        
        $mailView = new Zend_View();
        $mailView->title = $data['title'];
        $mailView->description = $data['desc'];
        $mailView->priority = $data['priority'];
        $mailView->link = $_SERVER['HTTP_HOST'] . $url->url(array(
            'language' => Zend_Registry::get('Zend_Translate')->getLocale(), 
            'module' => 'mobile', 
            'controller' => 'map', 
            'action' => 'index', 
            'job' => $data['id']
        ), null, true);
        
        $mailView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'tool' . DIR_SEP . 
            'views' . DIR_SEP . 'scripts' . DIR_SEP . 'jobs');
        
        if ($action == 'add') {
            $mailBody = $mailView->render('assign_mail' . GT_SCRIPT_EXT);
            $subject = $translate->_('taskAssigned') . ': ' . $data['title'];
        } else if ($action == 'edit') {
            $mailBody = $mailView->render('edit_mail' . GT_SCRIPT_EXT);
            $subject = $translate->_('taskEdited') . ': ' . $data['title'];
        }

        Giscuit_Mail::batch(array(
            array(
                'to' => $data['email'] ,
                'subject' => $subject , 
                'text' => $mailBody
            )
        ));
    }
    
    public function getForm($action) 
    {
        $form = new Giscuit_Form_Jobs($action);
        $form->setAttrib('enctype', 'multipart/form-data');
        $form->setAction($this->_helper->url->url(array(
            'language' => Zend_Registry::get('Zend_Translate')->getLocale() , 
            'module' => 'tool' , 
            'controller' => 'jobs'
        )));
        
        if ($action == 'edit') {
            $form->addRevertField();
        }

        return $form;
    }
    
    public function getWorkflowActions($jobs) 
    {
        $jobActionsMapTable = new Giscuit_Db_Table_JobsActionsMap();
        
        $workflowIds = array();
        foreach ($jobs as $job) {
            $workflowIds[] = $job['workflow_id'];
        }
            
        $workflowIds = array_unique($workflowIds);
        $possibleActions = $jobActionsMapTable->getItems(array(
            'whereWorkflow' => $workflowIds
        ))->toArray();
                
        $workflowActions = array();
        foreach ($possibleActions as $action) {
            $workflowActions[$action['workflow_id']][] = $action;
        }
            
        return $workflowActions;
    }

}