<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for Jobs communication
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Jobs extends Giscuit_Controller_Plugin_Abstract
{
    /**
     * Plugin display name
     *
     * @var string
     */
    public static $label = 'Tasks';
    
    protected $_translations = array(
        'en' => array(
            'jobs' => 'Tasks',
            'priority' => 'Priority',
            'completed' => 'Completed',
            'close' => 'Close',
            'enterComment' => 'Enter comment',
            'unknown' => 'Unknown',
            'showOnMap' => 'Show on map',
            'details' => 'Details',
            'description' => 'Description',
            'desc' => 'Description',
            'contact_address' => 'Contact address',
            'contact_name' => 'Contact name',
            'contact_phone' => 'Contact phone',
            'waiting' => 'Waiting',
            'ago' => 'ago',
            'accept' => 'Accept',
            'decline' => 'Decline',
            'checkin' => 'Check in',
            'checkout' => 'Check out',
            'high' => 'High',
            'medium' => 'Medium',
            'low' => 'Low',
            'deadline' => 'Deadline',
            'pending' => 'Pending',
            'in_progress' => 'In progress',
            'completed' => 'Completed',
            'declined' => 'Declined',
            'created' => 'Created',
            'assignTo' => 'Assign to',
            'in' => 'in',
            'title' => 'Title',
            'status' => 'Status',
            'workflow' => 'Workflow',
            'notify' => 'Notify',
            'dontNotify' => 'Do not notify',
            'notifyActions' => 'Notify upon new action',
            'notifyComplete' => 'Notify when task is accepted, declined, completed',
            'theme' => 'Theme',
            'parent' => 'Parent',
            'loadMore' => 'Load more',
            'worker' => 'Worker',
            'taskCompleted' => 'Task completed',
            'taskDeclined' => 'Task declined',
            'taskAccepted' => 'Task accepted',
            'hasNewAction' => 'has new action',
            'viewTaskGiscuit' => 'View this task in Giscuit',
            'taskEdited' => 'Task updated',
            'taskAssigned' => 'Task assigned',
            'revertStatusPending' => 'Update status to pending'
        ),
        'ro' => array(
            'jobs' => 'Sarcini',
            'priority' => 'Prioritate',
            'completed' => 'Terminate',
            'close' => 'Închide',
            'enterComment' => 'Scrieți comentariu',
            'unknown' => 'Necunoscută',
            'showOnMap' => 'Arată pe hartă',
            'details' => 'Detalii',
            'description' => 'Descriere',
            'desc' => 'Descriere',
            'contact_address' => 'Adresa contactului',
            'contact_name' => 'Numele contactului',
            'contact_phone' => 'Telefonul contactului',
            'waiting' => 'Așteaptă',
            'ago' => 'în urmă',
            'accept' => 'Accepta',
            'decline' => 'Refuză',
            'checkin' => 'Înregistrare',
            'checkout' => 'Plecare',
            'high' => 'Înaltă',
            'medium' => 'Medie',
            'low' => 'Joasă',
            'deadline' => 'Termen limită',
            'pending' => 'În așteptare',
            'in_progress' => 'În progres',
            'completed' => 'Finisat',
            'declined' => 'Refuzat',
            'created' => 'Creat',
            'assignTo' => 'Atribuie lui',
            'in' => 'peste',
            'title' => 'Titlu',
            'status' => 'Starea',
            'workflow' => 'Flux de lucru',
            'notify' => 'Anunțare',
            'dontNotify' => 'Fără anunțuri',
            'notifyActions' => 'Anunță despre acțiunea nouă',
            'notifyComplete' => 'Anunță cînd sarcina este acceptată, refuzată, finisată',
            'theme' => 'Culoare',
            'parent' => 'Părinte',
            'loadMore' => 'Încarcă mai multe',
            'worker' => 'Lucrător',
            'taskCompleted' => 'Sarcina finisată',
            'taskDeclined' => 'Sarcina refuzată',
            'taskAccepted' => 'Sarcina acceptată',
            'hasNewAction' => 'are acțiunea nouă',
            'viewTaskGiscuit' => 'Vizualizează sarcina în Giscuit',
            'taskEdited' => 'Sarcina actualizată',
            'taskAssigned' => 'Sarcina atribuită',
            'revertStatusPending' => 'Actualizează starea la "așteptare"'
        ),
        'ru' => array(
            'jobs' => 'Задания',
            'priority' => 'Приоритет',
            'completed' => 'Завершены',
            'close' => 'Закрыть',
            'enterComment' => 'Введите комментарий',
            'unknown' => 'Неизвестно',
            'showOnMap' => 'Показать на карте',
            'details' => 'Детали',
            'description' => 'Описание',
            'desc' => 'Описание',
            'contact_address' => 'Адрес контакта',
            'contact_name' => 'Имя контакта',
            'contact_phone' => 'Телефон контакта',
            'waiting' => 'Ожидает',
            'ago' => 'назад',
            'accept' => 'Принять',
            'decline' => 'Отклонить',
            'checkin' => 'Отметится',
            'checkout' => 'Выписатся',
            'high' => 'Высокий',
            'medium' => 'Средний',
            'low' => 'Низкий',
            'deadline' => 'Крайний срок',
            'pending' => 'В ожидании',
            'in_progress' => 'В процессе',
            'completed' => 'Завершено',
            'declined' => 'Отклонено',
            'created' => 'Создано',
            'assignTo' => 'Назначить',
            'in' => 'через',
            'title' => 'Название',
            'status' => 'Статус',
            'workflow' => 'Рабочий процесс',
            'notify' => 'Сообщать',
            'dontNotify' => 'Не сообщать',
            'notifyActions' => 'Сообщать о новых действиях',
            'notifyComplete' => 'Сообщить когда задача будет принята, отклонена, завершена',
            'theme' => 'Цвет',
            'parent' => 'Предок',
            'loadMore' => 'Загрузить еще',
            'worker' => 'Исполнитель',
            'taskCompleted' => 'Задание завершено',
            'taskDeclined' => 'Задание отклонено',
            'taskAccepted' => 'Задание принято',
            'hasNewAction' => 'получило новое действие',
            'viewTaskGiscuit' => 'Просмотреть задание в Giscuit',
            'taskEdited' => 'Задание обновлено',
            'taskAssigned' => 'Задание назначено',
            'revertStatusPending' => 'Обновить статус на "ожидание"'
        )
    );
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Jobs.js',
            'Layer/Jobs.js'
        );
    }
    
    /**
     * List of the javascript files to include for mobile
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
            'Mobile/Control/Jobs.js',
            'Mobile/Layer/Jobs.js' 
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        $rolesTable = new Giscuit_Db_Table_Roles();
        
        //Insert rules and roles for layer access
        $jobsWorkerRole = new stdClass();
        $jobsWorkerRole->key = 'jobs_worker';
        $rolesTable->addItem($jobsWorkerRole);
        
        $jobsDispatcherRole = new stdClass();
        $jobsDispatcherRole->key = 'jobs_dispatcher';
        $rolesTable->addItem($jobsDispatcherRole);
        
        $data = array('role' => 'jobs_dispatcher', 'resource' => 'tool_jobs_controller', 'privilege' => '', 'group' => 'controller');
        $rulesTable->insert($data);
        $data = array('role' => 'jobs_worker', 'resource' => 'mobile_jobs_controller', 'privilege' => '', 'group' => 'controller');
        $rulesTable->insert($data);
        
        //Create tables
        $schema = $newConfig->production->database->schema;

        $db->beginTransaction();
        try {
            $db->query("
            CREATE TABLE {$schema}.jobs
            (
                id serial NOT NULL,
                to_user_id integer,
                title text,
                \"desc\" text,
                contact_address text,
                contact_name text,
                contact_phone text,
                priority character varying(10),
                \"date\" timestamp with time zone,
                status character varying(15),
                the_geom geometry,
                from_user_id text,
                deadline timestamp with time zone,
                workflow_id integer,
                notify_actions character(1),
                notify_complete character(1),
                \"notify\" integer
            )");

            $db->query("ALTER TABLE ONLY {$schema}.jobs ADD CONSTRAINT jobs_pkey PRIMARY KEY (id)");
            
            $db->query("
            CREATE TABLE {$schema}.jobs_actions
            (
                id serial NOT NULL,
                user_id integer,
                job_id integer,
                comment text,
                \"date\" timestamp with time zone,
                the_geom geometry,
                action_map_id integer
            )");
            
            $db->query("ALTER TABLE ONLY {$schema}.jobs_actions ADD CONSTRAINT jobs_actions_pkey PRIMARY KEY (id)");
            $db->query("ALTER TABLE ONLY {$schema}.jobs_actions ADD CONSTRAINT jobs_actions_job_id_fkey
                 FOREIGN KEY (job_id) REFERENCES {$schema}.jobs (id) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE CASCADE");

            $db->query("
            CREATE TABLE {$schema}.jobs_actions_map
            (
                id serial NOT NULL,
                \"order\" integer,
                theme character(1),
                workflow_id integer,
                reserved integer,
                name_id integer,
                parent text
            )");
            
            $db->query("ALTER TABLE ONLY {$schema}.jobs_actions_map ADD CONSTRAINT jobs_actions_map_pkey PRIMARY KEY (id)");
            
            $db->query("
            CREATE TABLE {$schema}.jobs_actions_workflows
            (
                id serial NOT NULL,
                name_id integer
            )");
            
            $db->query("ALTER TABLE ONLY {$schema}.jobs_actions_workflows ADD CONSTRAINT jobs_actions_workflows_pkey PRIMARY KEY (id)");

            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }

        //Add default workflow
        $workflowsTable = new Giscuit_Db_Table_JobsActionsWorkflows();
        $textTable = new Giscuit_Db_Table_Text();
        $languages = $newConfig->production->language->toArray();
        
        $workflowData = $workflowsTable->addItem();
        $textRow = $textTable->find($workflowData['name_id'])->current();
        foreach ($languages as $langKey => $langValue) {
            $textRow->$langKey = 'Default';
        }
        $textRow->save();
        
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        $db = Zend_Registry::get('db');
        
        //Delete rules and roles
        $rulesTable = new Giscuit_Db_Table_Rules();
        $rolesTable = new Giscuit_Db_Table_Roles();
        
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'tool_jobs_controller');
        $rulesTable->delete($where);
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'mobile_jobs_controller');
        $rulesTable->delete($where);
        
        $where = $rolesTable->getAdapter()->quoteInto('key = ?', 'jobs_worker');
        $rolesTable->delete($where);
        
        $where = $rolesTable->getAdapter()->quoteInto('key = ?', 'jobs_dispatcher');
        $rolesTable->delete($where);
        
        $rolesTable->cleanupRoles();
        Giscuit_Acl::clearCache();
        
        //Drop tables
        $schema = $newConfig->production->database->schema;
        $db->beginTransaction();
        try {
            $db->query("DROP TABLE IF EXISTS {$schema}.jobs, {$schema}.jobs_actions, {$schema}.jobs_actions_map, {$schema}.jobs_actions_workflows CASCADE;");
            
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Parse settings in form
     *  
     * @return boolean
     */
    public function parseSettingsForm($form) {
        $form->addElement(
            'hidden',
            'jobsDivTag',
            array(
                'required' => false,
                'ignore' => true,
                'autoInsertNotEmptyValidator' => false,
                'decorators' => array(
                    array(
                        'HtmlTag', array(
                            'tag'   => 'div',
                            'id'    => 'jobsDiv'
                        )
                    )
                )
            )
        );
        $form->jobsDivTag->clearValidators();
        
        return true;
    }
    
    /**
     * Admin javascript script initialization code
     * 
     * @param $options Available options: 
     * @return string
     */
    public function getAdminJsInitCode(array $options = array()) 
    {
        return file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'static' . DIR_SEP . 'js' . DIR_SEP . 'giscuit' . DIR_SEP . 'Admin' . DIR_SEP . 'Jobs.js');
    }
    
    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams()
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        return array(
            'jobsPluginIsAllowedWorker' => $isAllowed->direct('mobile_jobs_controller') ? true : false,
            'jobsPluginIsAllowedDispatcher' => $isAllowed->direct('tool_jobs_controller') ? true : false
        );    
    }
}