<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: JobsActionsMap.php 2894 2012-08-01 14:23:23Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for Jobs Actions Map table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_JobsActionsMap extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'jobs_actions_map';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';
    
    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'name'
    );

    /**
     * Get items
     * 
     * @param array $options Available options:
     * <ul><li>whereWorkflow array - Array of workflow ids</li>
     * <li>withAllNames bool - Get name in all installed languages</li>
     * <li>order string - Sort column name and direction</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'parent' , 
            'order' , 
            'theme' ,
            'workflow_id' ,
            'reserved' ,
            'name_id'
        );
        
        $select->from($this, $fields);
        
        if (isset($options['whereId'])) {
            $select->where($this->_name . '.id = ?', $options['whereId']);
        }
        
        if (! empty($options['withMultilangFields'])) {
            $this->withMultilangFields($select, $options);
        } else {
            $this->joinMultilangField($select, 'name');
        }
        
        if (isset($options['whereWorkflow'])) {
            foreach ($options['whereWorkflow'] as $workflowId) {
                $select->orWhere($this->_name . '.workflow_id = ?', $workflowId);
            }
        }

        if(isset($options['order'])) {
            $select->order($options['order']);
        }
        
        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $config = Zend_Registry::get('config');
        
        $rows = $this->find($id);
        
        if ($rows->count() == 0) {
            throw new Giscuit_Exception("Invalid action id :" . $id);
        }
        
        $row = $rows->current();
        
        //Get column
        $column = $data->column;

        $row->$column = $data->data;
        
        return $row->save();
    }
    
    /**
     * Add item
     * 
     * @param integer $workflowId Action's workflow id
     * @return $data array - id, name_id and theme of the new element
     */
    public function addItem ($workflowId)
    {
        $newRow = $this->createRow();
        
        $textTable = new Giscuit_Db_Table_Text();

        $newRow->theme = 'c';
        $newRow->workflow_id = $workflowId;
        $newRow->parent = '';
            
        $newTextRow = $textTable->createRow();
        $newRow->name_id = $newTextRow->save();

        $data['name_id'] = $newRow->name_id;
        $data['theme'] = $newRow->theme;
        $data['id'] = $newRow->save();
        
        $newRow->order = $data['id'];
        $newRow->save();
        
        return $data;
    }
}
