<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Menu.php 1559 2011-08-03 10:57:49Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for incidents notifications table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */class Giscuit_Db_Table_Layer_IncidentsNotifications extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'incidents_notifications';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct ($options = array())
    {
        $config = Zend_Registry::get('config');

        $this->_schema = $config->database->schema . '_layers';
        $this->_mainSchema = $config->database->schema;
                
        parent::__construct($options);
    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $options <ul>Available options:
     * <li></li></ul>
     * @return void
     */
    public function addItem ($data, $options = array())
    {
        $newRow = $this->createRow();
        
        $newRow->user_id = $data['user_id'];
        //$newRow->email = $data['email'];
        
        return $newRow->save();
    }

    /**
     * Get items
     * 
     * @param array $options <ul>Available options:
     * <li>whereId integer - User id</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $config = Zend_Registry::get('config');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
        //    'email' ,
            'category_id' ,
            'user_id' ,
            'the_geom' => new Zend_Db_Expr('ST_AsText(' . $this->getAdapter()->quoteIdentifier($this->_name . '.the_geom') . ')')
        );
        
        $select->from($this, $fields);

        if (! empty($options['withEmail'])) {
            $select->joinLeft(array(
                'users' => $this->_mainSchema . '.users' ,
                ), "{$this->_name}.user_id = users.id" ,
                array('email', 'email_active')
            );
        }
        
        if (! empty($options['whereId'])) {
            $select->where($this->_name . '.user_id = ?', $options['whereId']);
        }
        
        if (! empty($options['whereGeomIntersects'])) {
            $theGeomField = $this->_name . '.the_geom';
            $geomFromText = "ST_GeomFromText('" . $options['whereGeomIntersects'] . "', {$config->map->srid})";
            $whereStm = "(ST_Intersects(" . $this->getAdapter()->quoteIdentifier($theGeomField) . ", " . $geomFromText . ")" .
                " OR {$this->_name}.the_geom IS NULL)";
            
            $select->where($whereStm);
        }
        
        if (! empty($options['whereCategoryId'])) {
            $whereStm = "({$this->_name}.category_id = " . $options['whereCategoryId'] . 
                " OR {$this->_name}.category_id IS NULL)";
            
            $select->where($whereStm);
        }

        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param array $data An array populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $config = Zend_Registry::get('config');
        
        $rows = $this->find($id);
        
        if ($rows->count() == 0) {
            throw new Giscuit_Exception("Invalid notification id :" . $id);
        }
        
        $row = $rows->current();
        
        if ($row->user_id != Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct()) {
            throw new Giscuit_Exception("Invalid user or column id");
        }
        
        //Get column
        $column = $data['column'];

        if ($column == 'the_geom') {
            if (empty($data['data'])) {
                $row->$column = null;
            } else {
                $row->$column = new Zend_Db_Expr("ST_Multi(ST_GeomFromText('" . $data['data'] . "', {$config->map->srid}))");
            }
        } else {
            $row->$column = $data['data'];
        }
        
        return $row->save();
    }
    
}
