<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayerSettings.php 3429 2013-06-05 07:28:44Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for layers settings table
 * 
 * Layer settings represent all the user specific layer configurations 
 * like view, select, edit, label (color, size, field), order  
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_LayerSettings extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'layer_settings';

    /**
     * Add item
     * 
     * @param Zend_Db_Table_Row $row Add row
     * @return integer The id of the new element
     */
    public function addItem ($row)
    {
        $layerDefaultsTable = new Giscuit_Db_Table_LayerDefaults();
        $defaultRow = $layerDefaultsTable->find($row['id']);
        
        if (! count($defaultRow)) {
            throw new Giscuit_Db_Table_Exception("No row found in layer_defaults with '{$row['id']}' id");
        } else {
            $defaultRow = $defaultRow->current()->toArray();
        }
        
        $newRow = $this->createRow();
        
        //label
        $newRow->label_size = $defaultRow['label_size'];
        $newRow->label_color = $defaultRow['label_color'];
        $newRow->label_fields = $defaultRow['label_fields'];
        
        //layer
        $newRow->view = $defaultRow['view'];
        $newRow->select = $defaultRow['select'];
        $newRow->edit = $defaultRow['edit'];
        $newRow->label = $defaultRow['label'];
        $newRow->order = $defaultRow['order'];
        $newRow->layer_id = $row['id'];
        $newRow->user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        //Save
        try {
            $pkey = $newRow->save();
        } catch (Exception $e) {
            
        }
        
        return $pkey;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return void
     */
    public function editSingleItem ($id, $data)
    {
        $boolFields = array(
            'view' , 
            'label' , 
            'select' , 
            'edit'
        );
        
        $row = $this->getCurrentUserRowByLayerId($id);
        
        $column = $data->column;
        
        if (in_array($column, $boolFields)) {
            //Only one layer can be editable
            if ($column == 'edit') {
                $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
                
                $editWhere = $this->getAdapter()->quoteInto('user_id = ?', $userId);
                
                $this->update(array(
                    'edit' => 'f'
                ), $editWhere);
            }
            
            $boolToChar = Zend_Controller_Action_HelperBroker::getStaticHelper('boolToChar');
            $data->value = $boolToChar->direct($data->value);
        }
        
        $row->$column = $data->value;
        
        try {
            $row->save();
        } catch (Exception $e) {

        }
    }

    /**
     * Cleanup label fields
     * 
     * @param string $oldName
     * @param string $newName
     * @return void
     */
    public function cleanupLabelFields ($oldName, $newName)
    {
        if (empty($newName)) {
            $value = '';
        } else {
            $value = ',' . $newName;
        }
        
        $data = array(
            'label_fields' => new Zend_Db_Expr("replace(label_fields, ',$oldName', '$value')")
        );
        $this->update($data, NULL);
        
        $data = array(
            'label_fields' => new Zend_Db_Expr("replace(label_fields, '$oldName', '$newName')")
        );
        $this->update($data, NULL);
    
    }

    /**
     * Get layer settings for the current user 
     * 
     * @param integer $id Id of the layer
     * @return Zend_Db_Table_Row
     */
    public function getCurrentUserRowByLayerId ($id)
    {
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $select = $this->select();
        $select->from($this);
        
        $select->where($this->_name . '.layer_id = ?', $id);
        $select->where($this->_name . '.user_id = ?', $userId);
        
        $row = $this->fetchRow($select);
        
        //Check user
        if (empty($row)) {
            throw new Giscuit_Db_Table_Exception("Invalid layer id ({$id}) or user id ({$userId})");
        }
        
        return $row;
    }

    /**
     * Get private layers
     * 
     * @return array
     */
    public function getPrivateItems ()
    {
        //
        $allowedFields = array('id', 'parent_id', 'path', 'name', 'class', 
            'metadata', 'edit_scale', 'order', 'js_isbase', 'js_file' ,
            'minscaledenom', 'maxscaledenom', 'view', 'select', 'edit' , 'query',
            'label', 'label_size', 'label_color', 'label_fields', 'default_field' ,
            'tolerance', 'search_scale', 'classlabel_active', 'max_gps_accuracy'
        );
        $layers = array();
        $categories = array();
        $isEdit = false;
        
        //
        $charToBool = Zend_Controller_Action_HelperBroker::getStaticHelper('charToBool');
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        //
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $mapOptions = array(
            'addEditLayer' => false , 
            'orderDirection' => 'asc' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withGroups' => true , 
            'withAcl' => true ,
            'withMetadata' => true ,
            'withFields' => true ,
        );
        $rows = $layersTable->getItems($mapOptions);
        $rows = $this->_removeEmptyContainers($rows);
                
        foreach ($rows as $row) {
            $newRow = array(
                'fields' => array()
            );
            
            //Populate
            foreach ($allowedFields as $field) {
                if(isset($row[$field])) {
                    $newRow[$field] = $row[$field];
                } else {
                    $newRow[$field] = NULL;
                }
            }
            
            if (! empty($newRow['class'])) {
                $newRow['view'] = $charToBool->direct($newRow['view']);
                $newRow['select'] = $charToBool->direct($newRow['select']);
                $newRow['label'] = $charToBool->direct($newRow['label']);
                $newRow['edit'] = $charToBool->direct($newRow['edit']);
                $newRow['query'] = $charToBool->direct($newRow['query']);
                $newRow['classlabel_active'] = $charToBool->direct($newRow['classlabel_active']);
                $newRow['js_isbase'] = $charToBool->direct($newRow['js_isbase']);
                
                if(is_subclass_of('Giscuit_Map_Layer_' . $newRow['class'], 'Giscuit_Map_Layer_JavaScript')) {
                    $newRow['js_layer'] = true;
                } else {
                    $newRow['js_layer'] = false;
                }

                //Raster layers have no labels and can't be selected
                $class = 'Giscuit_Map_Layer_' . $newRow['class'];
                
                if ((class_exists($class) && class_exists('Giscuit_Map_Layer_Vector') && is_subclass_of($class, 'Giscuit_Map_Layer_Vector'))) {
                    if(empty($row['default_field'])) {
                        $newRow['select'] = 'none';
                        $newRow['label'] = 'none';        
                    } else {
                        foreach ($row['fields'] as $value) {
                            $newRow['fields'][] = $value['name'];
                        }
                    }              
                } else if ((class_exists($class) && class_exists('Giscuit_Map_Layer_Raster') && is_subclass_of($class, 'Giscuit_Map_Layer_Raster')) ||
                    (class_exists('Giscuit_Map_Layer_Wms') && is_subclass_of($class, 'Giscuit_Map_Layer_Wms')) ||
                    (class_exists('Giscuit_Map_Layer_Javascript') && is_subclass_of($class, 'Giscuit_Map_Layer_Javascript'))
                    ) {
                    $newRow['label'] = 'none';
                    
                    if($row['query'] == 'f') {
                        $newRow['select'] = 'none';
                    }
                }
                
                //Check if the layer can be edited
                
                //Check user_id
                if($row['permissions_user_id'] == $userId) {
                    $isEdit = true;
                    
                //Check priv_editgeom
                } else if(strstr(',' . $row['permissions_editgeom_ids'], ',' . $userId . ',')) {
                    $isEdit = true;
                    
                //Check resource
                } else if (! empty($row['resource']) && 
                    (! empty($row['priv_editgeom']) || ! empty($row['priv_editattr']))
                    ) {
                    
                    if ($isAllowed->direct($row['resource'], $row['priv_editgeom']) || 
                        $isAllowed->direct($row['resource'], $row['priv_editattr'])
                        ) {
                        
                        $isEdit = true;
                    } else {
                        $newRow['edit'] = 'none';
                    }
                
                } else {
                    $newRow['edit'] = 'none';
                }
                
                $layers[] = $newRow;
            } else {
                $categories[] = $newRow;
            }
        
        }

        return array(
            'layers' => $layers , 
            'categories' => $categories , 
            'isEdit' => $isEdit
        );
 
    }

    /**
     * Get public layers
     * 
     * @return array
     */
    public function getPublicItems ()
    {
        //Base layers
        $baseTable = new Giscuit_Db_Table_Base();
        $baseRows = $baseTable->getItems(array(
            'whereActive' => 't'
        ));
        
        $baseLayers = array();
        foreach ($baseRows as $row) {
            $transitionEffect = NULL;
            
            if($row->parent_id == -1) {
                //Merged base
                $order = 60000 + $row->order;
                $transitionEffect = 'resize';                    
            } else {
                //Merged overlay
                $order = 50000 + $row->order;
            }
            
            $baseLayers[$row->id] = array(
                'id' => '_base' . $row->id , 
                'class' => '_base' . $row->id ,
                'layerClass' => '_base' . $row->id ,
                'selected' => $row->selected ,
                'format' => $row->format ,  
                'name' => $row->name ,
                'minscaledenom' => $row->minscaledenom , 
                'maxscaledenom' => $row->maxscaledenom ,
                'parent_id' => $row->parent_id ,
                'classes' => array() ,
                'ids' => array() ,
                'metadata' => array() ,
                'js_layer' => false ,
                'order' => $order ,
                'transitionEffect' => $transitionEffect,
                'merged' => false,
                'fields' => array()
            );
        }
       
        //Layers
        $mapOptions = array(
            'wherePublish' => 't' , 
            'whereActive' => 't' , 
            'withFields' => true , 
            'withPublish' => true , 
            'withGroups' => true ,
            'withMetadata' => true ,
        );
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $rows = $layersTable->getItems($mapOptions);
        
        $allowedFields = array('destination', 'id', 'name', 'tolerance' , 
            'order', 'selected', 'class', 'metadata', 
            'minscaledenom', 'maxscaledenom', 'format', 'parent_id',
            'search_scale', 'default_field'
        );
        $layers = array();
                
        //
        foreach($rows as $key => $row) {
            $newRow = array(
                'merged' => true,
                'fields' => array()
            );
            
            //Populate
            foreach ($allowedFields as $field) {
                if(isset($row[$field])) {
                    $newRow[$field] = $row[$field];
                } else {
                    $newRow[$field] = NULL;
                }
            }
            
            //
            if(empty($newRow['class'])) {
                $newRow['merged'] = false;
            }

            //
            if(empty($newRow['destination'])) {
                $newRow['destination'] = array();
            }
            
            $newRow['transitionEffect'] = '';
            $newRow['classes'] = $newRow['class'];
            $newRow['ids'] = $newRow['id'];
            
            //Vector
            if (! empty($newRow['class']) &&
                is_subclass_of('Giscuit_Map_Layer_' . $newRow['class'], 'Giscuit_Map_Layer_Vector')) {
                
                   foreach ($row['fields'] as $value) {
                       $newRow['fields'][] = $value['name'];
                   }
            }
            
            //JavaScript
            if(! empty($newRow['class']) &&
                is_subclass_of('Giscuit_Map_Layer_' . $newRow['class'], 'Giscuit_Map_Layer_JavaScript')){
                    
                $newRow['js_layer'] = true;
                
                //If JavaScript layer is a base layer force parent_id to -1
                if($row['js_isbase'] == 't') {
                    $newRow['parent_id'] = -1;
                }
            } else {
                $newRow['js_layer'] = false;
            }
           
                                    
            if(! empty($newRow['minscaledenom'])) {
                $newRow['metadata'] = array_merge(array('minscaledenom' => $newRow['minscaledenom']), $newRow['metadata']);
            }
            if(! empty($newRow['maxscaledenom'])) {
                $newRow['metadata'] = array_merge(array('maxscaledenom' => $newRow['maxscaledenom']), $newRow['metadata']);
            }
            
            
            foreach ($newRow['destination'] as $destination) {
                if($destination == 'overlay') {
                    $newRow['merged'] = false;
                }
                
                //Order for javascript base layers
                if($destination == 'base') {
                    $newRow['order'] = 60000 + $newRow['order'];
                    
                    $newRow['merged'] = false;    
                }
                
                if(! isset($baseLayers[$destination])) {
                    continue;
                }
                
                if($destination != 'overlay' && $destination != 'base') {
                    $baseLayers[$destination]['classes'][] = $newRow['class'];
                    $baseLayers[$destination]['ids'][] = $newRow['id'];
                }
                
                if(! empty($newRow['metadata'])) {
                    $baseLayers[$destination]['metadata'][] = $newRow['metadata'];
                }
            }
            
            //
            $newRow['layerClass'] = $newRow['class'];
            
            $layers[$newRow['id']] = $newRow;
        }
        
        return array(
            'layers' => $layers , 
            'isEdit' => false , 
            'baseLayers' => $baseLayers
        );
    }

    /**
     * Remove empty containers
     * 
     * @param Zend_Db_Table_Rowset $rows
     * @return Zend_Db_Table_Rowset
     */
    protected function _removeEmptyContainers ($rows)
    {
        $aRemove = true;
        while ($aRemove != false) {
            $aRemove = false;
            foreach ($rows as $aKey => $aValue) {             
                if (empty($aValue['class'])) {
                    $bRemove = true;
                    foreach ($rows as $bKey => $bValue) {
                        if ($bValue['parent_id'] == $aValue['id']) {
                            $bRemove = false;
                            break;
                        }
                    }
                    if ($bRemove == true) {
                        $aRemove = true;
                        unset($rows[$aKey]);
                    }
                }
            }
        }
        
        return $rows;
    }
    
    /**
     * Edit label settings
     * 
     * @param integer $id The id of the row to edit 
     * @param mixed $values Data
     * @return void
     */
    public function editItem ($id, $values)
    {
        if($values instanceof Zend_Form) {
            $values = $values->getValues();
        }

        //Parse
        if (is_array($values['label_fields'])) {
            $values['label_fields'] = implode(',', $values['label_fields']);
        } else {
            $values['label_fields'] = '';
        }
        
        //Only one layer can be editable
        if (isset($values['edit']) && $values['edit'] == 't') {
            $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
                
            $editWhere = $this->getAdapter()->quoteInto('user_id = ?', $userId);
                
            $this->update(array(
                'edit' => 'f'
            ), $editWhere);
        }
        
        //Get
        $row = $this->getCurrentUserRowByLayerId($id);
        
        //Check if label fields should be updated
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersRow = $layersTable->find($row->layer_id)->current();
        if($layersRow->classlabel_active == 't') {
            unset($values['label_fields']);
        }
        
        //Set
        foreach ($values as $valueKey => $value) {
            $row->$valueKey = $value; 
        }
        
        //Save
        $row->save();
    }
}
