<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: HTMLPurifier.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Filter
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see HTMLPurifier
 */
require_once 'HTMLPurifier/HTMLPurifier.standalone.php';

/**
 * Class for purifying HTML
 * 
 * @category   Giscuit
 * @package    Giscuit_Filter
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Filter_HTMLPurifier implements Zend_Filter_Interface
{

    /**
     * HtmlPurifier object placeholder
     * 
     * @var HTMLPurifier
     */
    protected $_htmlPurifier = null;

    /**
     * Sets filter options
     *
     * @param array $options Available options
     * @return void
     */
    public function __construct ($options = null)
    {
        if (Zend_Registry::isRegistered('htmlPurifier')) {
            $this->_htmlPurifier = Zend_Registry::get('htmlPurifier');
        } else {
            $config = HTMLPurifier_Config::createDefault();
            
            $config->set('HTML.DefinitionID', 'enduser-customize.html tutorial');
            $config->set('HTML.DefinitionRev', 1);
            /*
            $def = $config->getHTMLDefinition(true);
            $def->addAttribute('a', 'target', 'Enum#_blank,_self,_target,_top');
            */
            if ($def = $config->maybeGetRawHTMLDefinition()) {
                $def->addAttribute('a', 'target', 'Enum#_blank,_self,_target,_top');
            }
            
            $this->_htmlPurifier = new HTMLPurifier($config);
            
            Zend_Registry::set('htmlPurifier', $this->_htmlPurifier);
        }
    
    }

    /**
     * Defined by Zend_Filter_Interface
     *
     * Returns the string $value purifying the HTML
     * 
     * @param  string $value
     * @return string
     */
    public function filter ($value)
    {
        return $this->_htmlPurifier->purify($value);
    }

}