<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: ProtectImage.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Filter
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Filter_Interface
 */
require_once 'Zend/Filter/Interface.php';

/**
 * Class for watermarking images
 * 
 * @category   Giscuit
 * @package    Giscuit_Filter
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Filter_ProtectImage implements Zend_Filter_Interface
{

    /**
     * Sets filter options
     *
     * @param array $options Available options
     * @return void
     */
    public function __construct (array $options = array())
    {

    }

    /**
     * Defined by Zend_Filter_Interface
     * 
     * Watermark your image
     * 
     * Available options:
     * text => string - Watermark text;
     * dpi => integer - Image resolution;
     * 
     * @param mixed $img Image resource
     * @param array $options Available options
     * @return mixed 
     */
    public function filter ($img, $options = null)
    {
        $config = Zend_Registry::get('config');
        
        if (empty($options['dpi'])) {
            $options['dpi'] = 72;
        }            
            
        //Date
        $date = new Zend_Date();
        
        //User
        $auth = Zend_Auth::getInstance();
        if ($auth->hasIdentity()) {
            $name = $auth->getIdentity()->fname . ' ' . $auth->getIdentity()->lname;
        }
        
        //Copy
        if (! empty($options['text'])) {
            $text = $options['text'];
            $opacity = 25;
        } else {
            $text = $name . ' ' . $date->toString(GT_DATE_STYLE_PHP);
            $opacity = 15;
        }
        
        $noCopy = $this->_drawText($text, (12 * $options['dpi']) / 72);
        $this->_spamImage($img, $noCopy, $opacity);
               
        imagedestroy($noCopy);
        
        return $img;
    }

    /**
     * Draw $text on the image
     * 
     * @param string $text Text to draw
     * @param integer $fontSize Font size 
     * @return mixed
     */
    protected function _drawText ($text, $fontSize)
    {
        $angle = 0;

        //Get font path
        $fontPath = NULL;
        $contents = file(GT_CONFIG_PATH . DIR_SEP . 'fonts' . DIR_SEP . 'fonts.txt');
        foreach ($contents as $line) {
            $line = explode(' ', $line);
            if($line[0] == 'arial' || $line[0] == 'FreeSans') {
                $fontPath = $line[1];
                break;  
            }   
        }
        if(empty($fontPath)) {
            throw new Giscuit_Exception('Unable to find the path for arial or FreeSans font');
        }
        
        $noCopySize = imagettfbbox($fontSize, $angle, $fontPath, $text);
        
        $noCopyW = abs($noCopySize[2]) + abs($noCopySize[0]);
        $noCopyH = abs($noCopySize[7]) + abs($noCopySize[1]);
        
        $noCopy = imagecreatetruecolor($noCopyW, $noCopyH);
             
        imagefill($noCopy, 0, 0, imagecolorallocate($noCopy, 255, 255, 255));
        
        $fontColor = imagecolorallocate($noCopy, 0, 0, 0);
        imagettftext($noCopy, $fontSize, $angle, 0, $fontSize, $fontColor, $fontPath, $text);
       
        return $noCopy;
    }

    /**
     * Fill the image with text
     * 
     * @param mixed $img Input image resource
     * @param mixed $mask Text's image resource
     * @param integer $opacity Text's image opacity
     * @return mixed
     */
    protected function _spamImage ($img, $mask, $opacity = 100)
    {
        
        $swidth = imageSX($img);
        $sheight = imageSY($img);
        
        $owidth = imageSX($mask);
        $oheight = imageSY($mask);
        
        $offsetW = $swidth / 6;
        $offsetH = $sheight / 6;
        
        if($swidth < 750 || $sheight < 500) {
            imagecopymerge($img, $mask, $swidth / 2 - $owidth / 2, $sheight / 2 - $oheight / 2, 0, 0, $owidth, $oheight, $opacity);
        } else if($swidth > 750 || $sheight > 500) {
            imagecopymerge($img, $mask, $swidth / 2 - $owidth / 2, $sheight / 2 - $oheight / 2, 0, 0, $owidth, $oheight, $opacity);     
            imagecopymerge($img, $mask, 0 + $offsetW, 0 + $offsetH, 0, 0, $owidth, $oheight, $opacity);
            imagecopymerge($img, $mask, $swidth - $owidth - $offsetW, 0 + $offsetH, 0, 0, $owidth, $oheight, $opacity);
            imagecopymerge($img, $mask, 0 + $offsetW, $sheight - $oheight - $offsetH, 0 , 0, $owidth, $oheight, $opacity);
            imagecopymerge($img, $mask, $swidth - $owidth - $offsetW, $sheight - $oheight - $offsetH, 0, 0, $owidth, $oheight, $opacity);
        }

        return $img;
    }

}
