<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersBase.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Add base layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersBase extends Giscuit_Form_Abstract
{

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $languages = Zend_Registry::get('config')->language->toArray();
        
        $languageIdArray = array();
        
        foreach ($languages as $key => $language) {
            
            $languageIdArray[] = 'name_' . $key;
            
            $this->addElement('text', 'name_' . $key, array(
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    )
                ) , 
                'required' => false , 
                'label' => $language
            ));
            
        }

        $this->addDisplayGroup($languageIdArray, 'name');
        $nameSection = $this->getDisplayGroup('name');
        $nameSection->setLegend($translate->_('name'));
        
        $this->addElement('checkbox', 'active', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $this->addElement('checkbox', 'selected', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'selected' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Wms format
        $wmsFormatArr = array(
            'jpeg' => 'jpeg' ,
            'png' => 'png' , 
            'png24' => 'png24' 
        );
        $this->addElement('select', 'format', array(
            'required' => false , 
            'label' => 'format' , 
            'multiOptions' => $wmsFormatArr,
            'value' => 'jpeg'
        ));
        
        //Maxscaledenom
        $this->addElement('text', 'maxscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'maxscaledenom'
        ));
        
        //Minscaledenom
        $this->addElement('text', 'minscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'minscaledenom'
        ));
        
        //Parent id
        $this->addElement('select', 'parent_id', array(
            'required' => true , 
            'label' => 'container' 
        ));
        
        //Clear cache
        $this->addElement('button', 'clear_cache', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'clearCache' , 
            'id' => 'clearCache' ,
        ));
        
        $this->addElement('submit', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' ,
            'id' => 'submit_button' 
        ));
       
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */ 
    public function populate (array $values)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result = array();
        $result[-1] = $translate->_('baseLayer');
        $result[0] = '/';
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $rows = $layersTable->getItems($this->getOptions());
        
        foreach ($rows as $row) {
            
            if (empty($row['class'])) {
                $path = $row['path'];
                if (! empty($path) && $path != '/') {
                    $path = trim($path, '/');
                    $path = explode('/', $path);
                    foreach ($path as $key => $value) {
                        $path[$key] = $rows[$path[$key]]['name'];
                    }
                    $row['name'] = implode('/', $path);
                    $result[$row['id']] = '/' . $row['name'];
                }
            }

        }
        
        $parentId = $this->getElement('parent_id');
        $parentId->setMultiOptions($result);
        
        $result = $this->setDefaults($values);
        
        return $result;
    }
    
    /**
     * Get options
     * 
     * @return array
     */
    public function getOptions ()
    {
        $mapOptions = array(
            'orderDirection' => 'asc' , 
            'withGroups' => true
        );
        
        return $mapOptions;
    }

}