<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersEditWms.php 3442 2013-06-12 16:02:07Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Edit wms layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersEditWms extends Giscuit_Form_Abstract
{
    /**
     * Rules table
     * 
     * @var Giscuit_Db_Table_Rules
     */
    protected $_rulesTable;

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'save' , 
            'id' => 'layersSubmit'
        ));
        
        //Form view helper
        $this->addDecorators(array(
            array(
                'viewScript' , 
                array(
                    'viewScript' => 'forms' . DIR_SEP . 'layersEditWms' . GT_SCRIPT_EXT
                )
            )
        ));
    
    }

    /**
     * Add general section
     * 
     * @param integer $id Layer id
     * @return void
     */
    public function addGeneralSection ($id)
    {
        //Active
        $this->addElement('checkbox', 'active', array(
            'validators' =>  $this->checkboxValidator , 
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $containers = Giscuit_Db_Table_Layers::getContainers($id);
        
        $this->addElement('select', 'parent_id', array(
            'label' => 'container' , 
            'multiOptions' => $containers 
        ));
        
        //Uri
        $this->addElement('text', 'owsclient_uri', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        'min' => 8,
                        'max' => 300
                    )
                ),
                array(
                    'Uri'
                )
            ) , 
            'required' => true , 
            'label' => 'uri'
        ));
        
        //Image format
        $imageFormatArr = array(
            'png' => 'png' , 
            'png24' => 'png24' , 
            'jpeg' => 'jpeg'
        );
        $this->addElement('select', 'owsclient_image_format', array(
            'label' => 'imageFormat' , 
            'multiOptions' => $imageFormatArr ,
            'value' => 'jpeg'
        ));
        
        //Version
        $versionArr = array(
            '1.0.0' => '1.0.0' ,
            '1.0.7' => '1.0.7' ,
            '1.1.0' => '1.1.0' ,
            '1.1.1' => '1.1.1' , 
            '1.3.0' => '1.3.0'  
        );
        $this->addElement('select', 'owsclient_version', array(
            'label' => 'version' , 
            'multiOptions' => $versionArr ,
            'value' => '1.1.1' 
        ));
        
        //Layers
        $this->addElement('select', 'owsclient_layers', array(
            'required' => true ,
            'registerInArrayValidator' => false,
            'multiple' => true,
            'label' => 'layersUpdatedOnParamsChange'
        ));
        
        //Info format
        $infoFormatArr = array(
            'text/plain' => 'text/plain' ,
            'text/html' => 'text/html' 
        );
        $this->addElement('select', 'owsclient_info_format', array(
            'required' => true , 
            'label' => 'infoFormat' , 
            'multiOptions' => $infoFormatArr ,
            'value' => 'text/html'
        ));
        
        //Minscaledenom
        $this->addElement('text', 'minscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'minscaledenom'
        ));
        
        //Maxscaledenom
        $this->addElement('text', 'maxscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'maxscaledenom'
        ));
        
        //Opacity
        $this->addElement('text', 'opacity', array(
            'validators' => array(
                'Digits' , 
                array(
                    'Between' , 
                    false , 
                    array(
                        0 , 
                        100
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'opacity'
        ));
       
        //Tolerance
        $this->addElement('text', 'tolerance', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'tolerance'
        ));
        
        //Offsite
        $this->addElement('text', 'offsite', array(
            'filters' => array(
                    'StringTrim' , 
            ) , 
            'validators' => array(
                'Color'    
            ) , 
            'required' => false , 
            'label' => 'offsite'
        ));
        
        
        //Raster query
        $this->addElement('checkbox', 'query', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'query' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
     
    }

    /**
     * Add name section
     * 
     * @return void
     */
    public function addNameSection ()
    {
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($languages as $key => $language) {
            $this->addElement('text', 'name_' . $key, array(
                'filters' => array(
                    'HtmlSpecialChars'
                ) ,
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    ),
                ) , 
                'required' => false , 
                'label' => $language
            ));
        }
    }

    /**
     * Add default settings section
     * 
     * @return void
     */
    public function addSettingsSection ()
    {
        //View
        $this->addElement('checkbox', 'view', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'view' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Select
        $this->addElement('checkbox', 'select', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'select' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add permissions section
     * 
     * @return void
     */
    public function addPermissionsSection ()
    {
        //Resource
        $this->_rulesTable = new Giscuit_Db_Table_Rules();
        $resources = $this->_rulesTable->getItems(array(
            'getResourceList' => 'layer'
        ));
        $resources = $resources->toArray();
        $resourcesArr = array(
            '' => ''
        );
        foreach ($resources as $resource) {
            $resourcesArr[$resource['resource']] = $resource['resource'];
        }
        $this->addElement('select', 'resource', array(
            'label' => 'resource' , 
            'attribs' => array(
                'id' => 'menuResource'
            ) , 
            'multiOptions' => $resourcesArr
        ));
        
        //View privilege
        $this->addElement('select', 'priv_view', array(
            'required' => false , 
            'label' => 'priv_view' , 
            'attribs' => array(
                'id' => 'menuViewPrivilege'
            )
        ));
        
        //Ows privilege
        $this->addElement('select', 'priv_ows', array(
            'required' => false , 
            'label' => 'priv_ows' , 
            'attribs' => array(
                'id' => 'menuOwsPrivilege'
            )
        ));
    }
    
    /**
     * Add wms section
     * 
     * @return void
     */
    public function addWmsSection()
    {
        //Private
        $this->addElement('checkbox', 'wms_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wms_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add wfs section
     * 
     * @return void
     */
    public function addWfsSection()
    {
        //Private
        $this->addElement('checkbox', 'wfs_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wfs_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add wcs section
     * 
     * @return void
     */
    public function addWcsSection()
    {
        //Private
        $this->addElement('checkbox', 'wcs_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wcs_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Formats
        $formats = array_merge(array('' => ''), Giscuit_Map_Map::getOutputformatsList());
        $this->addElement('multiselect', 'wcs_formats', array(
            'required' => false , 
            'label' => 'outputFormats' ,
            'multiOptions' => $formats
        ));
    }
    
    /**
     * Add publish section
     * 
     * @return void
     */
    public function addPublishSection($className)
    {
        //Selected
        $this->addElement('checkbox', 'selected', array(
            'validators' => $this->checkboxValidator , 
            'value' => 'f' , 
            'required' => true , 
            'label' => 'publish_selected' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Destination
        $destinationArr = array(
            'overlay' => 'overlay' 
        );
        
        $baseTable = new Giscuit_Db_Table_Base();
        $baseRows = $baseTable->getItems();
        
        foreach($baseRows as $row) {
            $destinationArr[$row->id] = $row->name;
        }
        
        $this->addElement('multiselect', 'destination', array(
            'label' => 'publish_destination' , 
            'isArray' => true , 
            'multiOptions' => $destinationArr
        ));
        
        //Wms format
        $wmsFormatArr = array(
            'png' => 'png' , 
            'png24' => 'png24' , 
            //'jpeg' => 'jpeg'
        );
        $this->addElement('select', 'format', array(
            'label' => 'format' , 
            'multiOptions' => $wmsFormatArr
        ));
        
        //Clear cache
        $clearCacheDisabled = array();
        if(! file_exists(GT_PUBLIC_PATH . DIR_SEP . 'tiles' . DIR_SEP . $className)) {
            $clearCacheDisabled['disabled'] = 'disabled';    
        }
        $this->addElement('button', 'clear_cache', array_merge($clearCacheDisabled, array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'clearCache' , 
            'id' => 'clearCache' ,
        )));
        
    }
    
    /**
     * Add wms authentication section
     * 
     * @return void
     */
    public function addAuthSection()
    {

        //owsclient_auth_type
        $typeArr = array(
            '' => '' ,
            'any' => 'any' , 
            'basic' => 'basic' , 
            'ntlm' => 'ntlm' ,
        );
        $this->addElement('select', 'owsclient_auth_type', array(
            'label' => 'type' , 
            'multiOptions' => $typeArr
        ));
        
        //         
        $this->addElement('text', 'owsclient_auth_username', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 100
                    )
                )
            ) , 
            'required' => false , 
            'label' => 'username'
        ));
        
        //
        $this->addElement('password', 'owsclient_auth_password', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 100
                    )
                )
            ) ,
            'required' => false ,
            'renderPassword' => true,
            'label' => 'password'
        ));
        
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        //Privledges
        if (! empty($values['resource'])) {
            $privileges = $this->_rulesTable->getItems(array(
                'getPrivilegeList' => $values['resource']
            ));
            $privilegesArr = array(
                '' => ''
            );
            foreach ($privileges as $privilege) {
                $privilegesArr[$privilege['privilege']] = $privilege['privilege'];
            }
            
            $this->getElement('priv_view')->setMultiOptions($privilegesArr);
            $this->getElement('priv_ows')->setMultiOptions($privilegesArr);
            
        }
        
        $result = $this->setDefaults($values);
        
        return $result;
       
    }
}