<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Incidents.php 2182 2012-01-20 12:01:13Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Incidents layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Layer_Incidents extends Giscuit_Form_Layer_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        //Set method and id
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtIncidentsForm'
        ));
        
        //Title
        $this->addElement('text', 'title', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        0 , 
                        100
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'title'
        ));
        
        //Description
        $this->addElement('textarea', 'description', array(
            'filters' => array(
                'StringTrim'
            ) , 
            'required' => true , 
            'rows' => 5,
            'cols' => 40,
            'label' => 'description'
        ));
        
        $incidentsCategoryTable = new Giscuit_Db_Table_Layer_IncidentsCategory();
        $categoryItems = $incidentsCategoryTable->getItems(array('withName' => true))->toArray();
        $categories = array();
        for ($i = 0; $i < count($categoryItems); $i++) {
          $categories[$categoryItems[$i]['id']] = $categoryItems[$i]['name'];
        }
        
        //Category
        $this->addElement('select', 'category_id', array(
            'required' => true , 
            'label' => 'category' , 
            'multiOptions' => $categories
        ));
        
        //First name
        $this->addElement('text', 'fname', array(
            'filters' => array(
                'StringTrim',
                'HtmlEntities',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        0 , 
                        30
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'fname'
        ));
        
        //Last name
        $this->addElement('text', 'lname', array(
            'filters' => array(
                'StringTrim',
                'HtmlEntities',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        0 , 
                        30
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'lname'
        ));
        
        //Email
        $this->addElement('text', 'email', array(
            'filters' => array(
                'StringTrim',
                'StringtoLower'
            ) , 
            'validators' => array(
                array(
                    'EmailAddress'
                )
            )
             , 
            'required' => true , 
            'label' => 'email'
        ));
        
        //Image
        $this->addElement('file', 'image', array(
            'validators' => array(
                'Count' => array(
              'files' => 1
            ),
            'FilesSize' => array(
              'min' => 1,
              'max' => '5MB'
            ),
            'Extension' => array(
              'jpg',
              'png',
              'gif'
            )
            ), 
            'required' => false , 
            'label' => 'image'
        ));
    
    }
}