<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Class.php 3033 2012-10-17 15:29:00Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Map_Abstract
 */
require_once 'Giscuit/Map/Abstract.php';

/**
 * Class for abstract mapserver object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Map_Class extends Giscuit_Map_Abstract
{
    
    public $legend = true;
    
    /**
     * Class constructor
     *
     * @param mixed $layer Layer object
     * @param string $name Class name
     * @return void
     */
    public function __construct ($layer, $name)
    {
        if(empty($name)) {
            throw new Giscuit_Exception('Class name is empty');
        }
        
        $this->_obj = ms_newclassobj($layer->getMsObj());
        $this->parent = $layer;     
        
        $this->set('name', $name);
                
        $this->set('status', MS_ON);
        
        $this->parent->children[$this->name] = $this;
        
    }
    
    /**
     * Saves icon image to a file
     * 
     * @return void
     */ 
    public function createLegend ()
    {
        $translate = Zend_Registry::get('Zend_Translate');

        if($this->parent->name == 'Copyright') {
            return;
        }
        
        //Image path
        $fileName = $this->parent->name . '_' . $this->name . '.png';
        $imagePath = GT_PUBLIC_PATH . DIR_SEP . 'legend' . DIR_SEP.  $fileName;
        
        //Construct icon
        if (! file_exists($imagePath)) {
            $classIcon = $this->createLegendIcon(30, 20);
            $iconUrl = $classIcon->saveImage($imagePath);
        }
        
        $settings = $this->parent->getSettings();
        $map = $this->parent->parent;
        
        if (! $this->legend || ! isset($settings['name'])) {
            return;
        }
        
        if(! isset($map->legendArr[$settings['class']]['name'])) {
            $map->legendArr[$settings['class']]['name'] = $settings['name'];
        }
        $map->legendArr[$settings['class']][] = array(
            'file' => $fileName , 
            'name' => $translate->_($this->name)
        );
        
        $map->legendArr[$settings['class']] = $this->parent->parseLegend($map->legendArr[$settings['class']]);
    }

    /**
     * Returns a style from the class given an index
     * 
     * @param integer $index Index value (0 = first style)
     * @return Giscuit_Map_Style
     */
    public function getStyle($index) {
        if(isset($this->children[$index])) {
            return $this->children[$index];
        } else {
            throw new Giscuit_Exception("Style ({$index}) was not found in the haystack.");
        }
    }
    
    /**
     * Remove a style from the map
     * 
     * @param integer $index Index value (0 = first style)
     * @return Giscuit_Map_Style Returns the removed style on success, else null.
     */
    public function removeStyle($index) {
        if(isset($this->children[$index])) {
            $this->getMsObj()->deleteStyle($index);
            $this->numstyles--;
            
            unset($this->children[$index]);    
            $this->children = array_values($this->children);
        } else {
            throw new Giscuit_Exception("Style ({$index}) was not found in the haystack.");    
        }
    }
    
    /**
     * Delete
     * 
     * @return void
     */
    public function delete()
    {
        for($i = 0; $i < $this->parent->numclasses; $i++) {
            if($this->parent->getClass($i)->name == $this->name &&
                in_array($this->name, array_keys($this->parent->children))) {
                    
                $this->parent->getMsObj()->removeClass($i);
                unset($this->parent->children[$this->name]);        
            } else {
                throw new Giscuit_Exception("Class ({$this->name}) was not found in the haystack.");
            }
        }
    }
    
    /**
     * Set the expression string for the class object.
     * 
     * @param string Expression
     * @return integer
     */
    public function setExpression($value) {
        return $this->getMsObj()->setExpression('(' . $value . ')');
    }

}