<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Masking.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for masking layer
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Map_Layer_Masking extends Giscuit_Map_Layer_Polygon
{
    public function __construct ($map, $settings)
    {
        parent::__construct($map, $settings);
        
        if ($map->getMetaData('ows_enable_request') == '*') {
            $this->setMetaData('wms_enable_request', '!GetCapabilities');
            $this->set('status', MS_DEFAULT);
        }
        
        $config = Zend_Registry::get('config');
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper("isAllowed");
        $maskingPlugin = Zend_Controller_Front::getInstance()->getPlugin("Giscuit_Controller_Plugin_Masking");
           
        if (! $isAllowed->direct("admin")) {
            $geometry = Giscuit_Map_Map::extentToWktPolygon($config->map->extent->toArray());
            
            $masksResult = $maskingPlugin->getCurrentMasks("role_view");
            
            if ($masksResult !== false) {
                //Create class and style
                $class = new Giscuit_Map_Class($this, "Mask");
    
                $defaultStyle = ms_newStyleObj($class->getMsObj());
                $defaultStyle->color->setRGB(255, 255, 255);

                $mapWindowShape = ms_shapeObjFromWkt($geometry);
                
                //Calculate difference between shapes
                foreach ($masksResult as $mask) {
                    $maskGeom = ms_shapeObjFromWkt($mask["the_geom"]);
                    $difference = $mapWindowShape->difference($maskGeom);
                    if (! $difference) {
                        return;
                    }
                    $mapWindowShape = $difference;
                }
                
                $this->addFeature($mapWindowShape);
            }
        }
    }
}