/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Coords.js 2744 2012-06-12 07:27:23Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if (gt.config.layout == 'desktop') {
            gt.control.coords = new Giscuit.Control.Coords();
        }
    }
});

/**
 * Class: Giscuit.Control.Coords
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Coords = OpenLayers.Class(Giscuit.Control, {
    
    importButton: null,
    
    clearButton: null,
    
    layer: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        if (! gt.pluginParams.coordsPluginIsAllowed) {
            return;
        }
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon('coords.png'), 
            id: "gtControlCoords", 
            onclick: {fn: function() {
                gt.control.coords.activate();
            }},
            title: gt.translate._('coords'), 
            container: "gtMapControls" 
        });

    },
    
    activate: function() {
        //Deactivate all active controls
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
    
        if (activated == true) {
            //Check button
            this.button.set('checked', true);
            
            if (this.panel == null) {
                var html = this.getLoadingImage();
                
                gt.accord.addPanel({
                    label: gt.translate._('quickImport'),
                    content: html
                });
                this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
                this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.coords.pkg.html');
                
                gt.accord.openPanel(this.getPanelIndex());
                
                this.create();
            } else {
                gt.accord.openPanel(this.getPanelIndex());
            }
        } else {
            gt.accord.openPanel(this.getPanelIndex());
        }
        
        return activated;
    },
    
    create: function() {
        
        //Add css
        var css = '.gtCoordsSeparator {border-bottom: 1px solid #CCC; padding-bottom: 5px; margin-bottom: 5px;}' +
            '.gtCoordsLabel {padding-right: 20px;}';
            
        var styleSheet = new YAHOO.util.StyleSheet(css, this.CLASS_NAME);
        
        //Add html
        //var projectionLink = gt.map.projection.toString().replace(':', '::');
        //var projectionName = gt.translate._('mapProjection').replace('%1$s', projectionLink).replace(/%2\$s/g, gt.map.projection);

        var Dom = YAHOO.util.Dom;
        var html = '';
        
        html += '<div class="gtCoordsSeparator">' +
        '<label class="gtCoordsLabel">' +
            '<input id="gtCoordsTypeText" class="gtCoordsTypeCheckbox" type="radio" value="text" name="gtCoordsType" checked="checked">' +
            gt.translate._('text') +
        '</label>' +
        '<label class="gtCoordsLabel">' +
            '<input id="gtCoordsTypeGpx" class="gtCoordsTypeCheckbox" type="radio" name="gtCoordsType" value="gpx">' +
            gt.translate._('gpx') +
        '</label>' +
        '<label class="gtCoordsLabel">' +
            '<input id="gtCoordsTypeKml" class="gtCoordsTypeCheckbox" type="radio" name="gtCoordsType" value="kml">' +
            gt.translate._('kml') +
        '</label>' +
        '<label class="gtCoordsLabel">' +
            '<input id="gtCoordsTypeCsv" class="gtCoordsTypeCheckbox" type="radio" name="gtCoordsType" value="csv">' +
            gt.translate._('csv') +
        '</label>' +
        '</div>' +
        
        '<div id="gtCoordsImportContent">' +
            '<div id="gtCoordsImportContentText">' +
                '<form id="gtCoordsImportTextForm"><dl class="zend_form">' +
                '<dt><label for="gtCoordsImportX">' + gt.translate._('xCoord') + '</label></dt>' +
                '<dd><input id="gtCoordsImportX" type="text"></dd>' +
                '<dt><label for="gtCoordsImportY">' + gt.translate._('yCoord') + '</label></dt>' +
                '<dd><input id="gtCoordsImportY" type="text"></dd>' +
                '<dt><label for="gtCoordsImportLabel">' + gt.translate._('label') + '</label></dt>' +
                '<dd><input id="gtCoordsImportLabel" type="text"></dd>' +
                '</form>' +
            '</div>' +
            '<div id="gtCoordsImportContentGpx" style="display: none;">' +
                '<form id="gtCoordsImportGpxForm"><dl class="zend_form">' +
                '<dt><label for="gtCoordsImportGpxFile">' + gt.translate._('selectFile') + '</label></dt>' +
                '<dd><input id="gtCoordsImportGpxFile" type="file" name="file"></dd>' +
                '</form>' +
            '</div>' +
            '<div id="gtCoordsImportContentKml" style="display: none;">' +
                '<form id="gtCoordsImportKmlForm"><dl class="zend_form">' +
                '<dt><label for="gtCoordsImportKmlFile">' + gt.translate._('selectFile') + '</label></dt>' +
                '<dd><input id="gtCoordsImportKmlFile" type="file" name="file"></dd>' +
                '</form>' +
            '</div>' +
            '<div id="gtCoordsImportContentCsv" style="display: none;">' +
                '<form id="gtCoordsImportCsvForm"><dl class="zend_form">' +
                '<dt><label for="gtCoordsImportCsvFile">' + gt.translate._('selectFile') + '</label></dt>' +
                '<dd><input id="gtCoordsImportCsvFile" type="file" name="file"></dd>' +
                '</form>' +
                '<div style="color: #CC1100;">' + gt.translate._('csvFormatWarning') + '</div><br>' +
            '</div>' +
            '<div id="gtCoordsImportContentProj">' +
                '<form id="gtCoordsImportProjForm"><dl class="zend_form">' +
                '<dt><label>' + gt.translate._('coordinatesIn') + ':</label></dt>' +
                '<dd><label><input id="gtCoordsProjTypeWgs" type="radio" name="gtCoordsProjType" value="EPSG:4326" checked="checked">' + gt.translate._('wgs') + '</label><br>' +
                '<label><input id="gtCoordsProjTypeMapProj" type="radio" name="gtCoordsProjType" value="mapProj">' + gt.map.projection + '</label></dd>' +
                '</form>' +
            '</div>' +
        '</div>' +
        
        '<button type="button" id="gtCoordsImportButton">' + gt.translate._('import') + '</button>' +
        '<button type="button" id="gtCoordsClearButton">' + gt.translate._('clear') + '</button>';
        
        gt.accord.setBody(this.getPanelIndex(), html);
        
        this.importButton = new YAHOO.widget.Button('gtCoordsImportButton');
        this.clearButton = new YAHOO.widget.Button('gtCoordsClearButton');
        
        //Layer
        this.addLayer();
        
        //Events
        var importType = YAHOO.util.Dom.getElementsByClassName('gtCoordsTypeCheckbox');
        YAHOO.util.Event.addListener(importType, "click", function (evt) {
            Dom.get('gtCoordsImportContentText').style.display = 'none';
            Dom.get('gtCoordsImportContentGpx').style.display = 'none';
            Dom.get('gtCoordsImportContentKml').style.display = 'none';
            Dom.get('gtCoordsImportContentCsv').style.display = 'none';
            Dom.get('gtCoordsImportContentProj').style.display = 'none';
            if (this.value == 'text') {
                Dom.get('gtCoordsImportContentText').style.display = 'block';
                Dom.get('gtCoordsImportContentProj').style.display = 'block';
            } else if (this.value == 'gpx') {
                Dom.get('gtCoordsImportContentGpx').style.display = 'block';
            } else if (this.value == 'kml') {
                Dom.get('gtCoordsImportContentKml').style.display = 'block';
            } else if (this.value == 'csv') {
                Dom.get('gtCoordsImportContentCsv').style.display = 'block';
                Dom.get('gtCoordsImportContentProj').style.display = 'block';
            }
        });
        
        this.importButton.on("click", function () {
            if (Dom.get('gtCoordsTypeText').checked) {
                this.addFromText();
            } else if (Dom.get('gtCoordsTypeGpx').checked) {
                this.addFromGpx();
            } else if (Dom.get('gtCoordsTypeKml').checked) {
                this.addFromKml();
            } else if (Dom.get('gtCoordsTypeCsv').checked) {
                this.addFromCsv();
            }
        }, null, this);
        
        this.clearButton.on("click", function () {
            this.layer.removeAllFeatures();
        }, null, this);
        
    },
    
    addFromText: function() {
        
        var x = parseFloat(OpenLayers.Util.getElement('gtCoordsImportX').value);
        var y = parseFloat(OpenLayers.Util.getElement('gtCoordsImportY').value);
        var label = OpenLayers.Util.getElement('gtCoordsImportLabel').value;
        var pointGeom = new OpenLayers.Geometry.Point(x, y);
        
        if (OpenLayers.Util.getElement('gtCoordsProjTypeWgs').checked) {
            var wgs = new OpenLayers.Projection("EPSG:4326");
            pointGeom.transform(wgs, gt.map.projection);
        }

        var feature = new OpenLayers.Feature.Vector(pointGeom, {name: label});
        this.layer.addFeatures([feature]);
        
    },
    
    addFromGpx: function() {
        var fileElement = OpenLayers.Util.getElement("gtCoordsImportGpxFile");
        if (! fileElement.value) {
            return;
        }
        var format = new OpenLayers.Format.GPX({
            'internalProjection': gt.map.projection,
            'externalProjection': new OpenLayers.Projection("EPSG:4326")
        });
        this.addFeatures(fileElement, format);
    },
    
    addFromKml: function() {
        var fileElement = OpenLayers.Util.getElement("gtCoordsImportKmlFile");
        if (! fileElement.value) {
            return;
        }
        var format = new OpenLayers.Format.KML({
            'internalProjection': gt.map.projection,
            'externalProjection': new OpenLayers.Projection("EPSG:4326")
        });
        this.addFeatures(fileElement, format);
    },
    
    addFromCsv: function() {
        var fileElement = OpenLayers.Util.getElement("gtCoordsImportCsvFile");
        if (! fileElement.value) {
            return;
        }
        var format = new OpenLayers.Format.CSW();
        this.addFeatures(fileElement, format);
    },
    
    addFeatures: function(fileElement, format) {
        var self = this;
        
        if (fileElement.files && typeof(FileReader) !== "undefined") {
            var file = fileElement.files[0];
            var reader = new FileReader();
            
            reader.readAsText(file, "UTF-8");
            reader.onload = function (evt) {
                var features = format.read(evt.target.result);
                
                if (features && features.length) {
                    var bounds = new OpenLayers.Bounds();
                    for(var i = 0; i < features.length; ++i) {
                        bounds.extend(features[i].geometry.getBounds());
                    }
                    
                    self.layer.addFeatures(features);
                    gt.map.zoomToExtent(bounds);
                } else {
                    debug.warn('No features loaded');
                }
            };
            reader.onerror = function (evt) {
                debug.warn('Error loading file');
            };
        } else {
            //File API not supported
            this.uploadFile(fileElement, format);
        }
        
    },
    
    uploadFile: function(fileElement, format) {
        var self = this;
        
        var submitCallback = {
            upload: function(o) {
                this.importButton.set('disabled', false);
                
                if (o.responseXML && format.CLASS_NAME != "OpenLayers.Format.CSW") {
                    var fileContents = o.responseXML;
                } else {
                    var fileContents = o.responseText;
                }
                var features = format.read(fileContents);
                
                if (features && features.length) {
                    var bounds = new OpenLayers.Bounds();
                    for(var i = 0; i < features.length; ++i) {
                        bounds.extend(features[i].geometry.getBounds());
                    }
                    
                    self.layer.addFeatures(features);
                    gt.map.zoomToExtent(bounds);
                } else {
                    debug.warn('No features loaded');
                }
            },
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        this.importButton.set('disabled', true);
        var form = fileElement.parentElement.parentElement.parentElement;
        YAHOO.util.Connect.setForm(form, true);
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/coords", submitCallback);
    },
    
    addLayer: function() {
        if (! this.layer) {
            var defaultStyle = OpenLayers.Util.applyDefaults({
                label : "${name}\n" ,
                labelAlign: "cb"
            }, OpenLayers.Feature.Vector.style["default"]);
            
            this.layer = new OpenLayers.Layer.Vector("Coords", {
                styleMap: new OpenLayers.StyleMap({
                    'default':defaultStyle
                })
            });
            gt.control.layers.addLayer(this.layer, {name: gt.translate._('quickImport')}, 'overlay');
        }
    },
    
    clear: function() {
        //Buttons
        this.importButton.destroy();
        this.importButton = null;
        
        this.clearButton.destroy();
        this.clearButton = null;
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    CLASS_NAME: "Giscuit.Control.Coords"
});



OpenLayers.Format.CSW = OpenLayers.Class(OpenLayers.Format, {

    defaultStyle: null,

    extractStyles: true,

    initialize: function(options) {
        options = options || {};
        
        OpenLayers.Format.prototype.initialize.apply(this, [options]);
    }, 

    read: function(doc) {
        cswArray = this.CSVToArray(doc);
        
        var features = [];

        for (var i = 0; i < cswArray.length; i++) {
            var record = cswArray[i];
            var pointGeom = new OpenLayers.Geometry.Point(record[0], record[1]);

            if (OpenLayers.Util.getElement('gtCoordsProjTypeWgs').checked) {
                var wgs = new OpenLayers.Projection("EPSG:4326");
                pointGeom.transform(wgs, gt.map.projection);
            }

            var feature = new OpenLayers.Feature.Vector(pointGeom, {name: record[2]});
            features.push(feature);
        }
        
        return features;
    },
    
    CSVToArray: function(strData, strDelimiter) {
        strDelimiter = (strDelimiter || ",");
        var objPattern = new RegExp(("(\\" + strDelimiter + "|\\r?\\n|\\r|^)" + '(?:"([^"]*(?:""[^"]*)*)"|' + '([^"\\' + strDelimiter + "\\r\\n]*))"), "gi");
        var arrData = [
            []
        ];
        var arrMatches = null;
        while (arrMatches = objPattern.exec(strData)) {
            var strMatchedDelimiter = arrMatches[1];
            if (strMatchedDelimiter.length && (strMatchedDelimiter != strDelimiter)) {
                arrData.push([]);
            }
            if (arrMatches[2]) {
                var strMatchedValue = arrMatches[2].replace(new RegExp('""', "g"), '"');
            } else {
                var strMatchedValue = arrMatches[3];
            }
            arrData[arrData.length - 1].push(strMatchedValue);
        }
        return (arrData);
    },

    CLASS_NAME: "OpenLayers.Format.CSW" 
});

//OpenLayers.Format.GPX 2.12
OpenLayers.Format.GPX=OpenLayers.Class(OpenLayers.Format.XML,{defaultDesc:"No description available",extractWaypoints:!0,extractTracks:!0,extractRoutes:!0,extractAttributes:!0,namespaces:{gpx:"http://www.topografix.com/GPX/1/1",xsi:"http://www.w3.org/2001/XMLSchema-instance"},schemaLocation:"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd",creator:"OpenLayers",initialize:function(a){this.externalProjection=new OpenLayers.Projection("EPSG:4326");OpenLayers.Format.XML.prototype.initialize.apply(this,
[a])},read:function(a){"string"==typeof a&&(a=OpenLayers.Format.XML.prototype.read.apply(this,[a]));var c=[];if(this.extractTracks)for(var b=a.getElementsByTagName("trk"),d=0,e=b.length;d<e;d++){var f={};this.extractAttributes&&(f=this.parseAttributes(b[d]));for(var g=this.getElementsByTagNameNS(b[d],b[d].namespaceURI,"trkseg"),h=0,j=g.length;h<j;h++){var k=this.extractSegment(g[h],"trkpt");c.push(new OpenLayers.Feature.Vector(k,f))}}if(this.extractRoutes){e=a.getElementsByTagName("rte");b=0;for(d=
e.length;b<d;b++)f={},this.extractAttributes&&(f=this.parseAttributes(e[b])),g=this.extractSegment(e[b],"rtept"),c.push(new OpenLayers.Feature.Vector(g,f))}if(this.extractWaypoints){a=a.getElementsByTagName("wpt");b=0;for(e=a.length;b<e;b++)f={},this.extractAttributes&&(f=this.parseAttributes(a[b])),d=new OpenLayers.Geometry.Point(a[b].getAttribute("lon"),a[b].getAttribute("lat")),c.push(new OpenLayers.Feature.Vector(d,f))}if(this.internalProjection&&this.externalProjection){f=0;for(a=c.length;f<
a;f++)c[f].geometry.transform(this.externalProjection,this.internalProjection)}return c},extractSegment:function(a,c){for(var b=this.getElementsByTagNameNS(a,a.namespaceURI,c),d=[],e=0,f=b.length;e<f;e++)d.push(new OpenLayers.Geometry.Point(b[e].getAttribute("lon"),b[e].getAttribute("lat")));return new OpenLayers.Geometry.LineString(d)},parseAttributes:function(a){var c={};a=a.firstChild;for(var b,d;a;){if(1==a.nodeType&&a.firstChild&&(b=a.firstChild,3==b.nodeType||4==b.nodeType))d=a.prefix?a.nodeName.split(":")[1]:
a.nodeName,"trkseg"!=d&&"rtept"!=d&&(c[d]=b.nodeValue);a=a.nextSibling}return c},write:function(a,c){a=OpenLayers.Util.isArray(a)?a:[a];var b=this.createElementNS(this.namespaces.gpx,"gpx");b.setAttribute("version","1.1");b.setAttribute("creator",this.creator);this.setAttributes(b,{"xsi:schemaLocation":this.schemaLocation});c&&"object"==typeof c&&b.appendChild(this.buildMetadataNode(c));for(var d=0,e=a.length;d<e;d++)b.appendChild(this.buildFeatureNode(a[d]));return OpenLayers.Format.XML.prototype.write.apply(this,
[b])},buildMetadataNode:function(a){for(var c=["name","desc","author"],b=this.createElementNSPlus("gpx:metadata"),d=0;d<c.length;d++){var e=c[d];if(a[e]){var f=this.createElementNSPlus("gpx:"+e);f.appendChild(this.createTextNode(a[e]));b.appendChild(f)}}return b},buildFeatureNode:function(a){var c=a.geometry,c=c.clone();this.internalProjection&&this.externalProjection&&c.transform(this.internalProjection,this.externalProjection);if("OpenLayers.Geometry.Point"==c.CLASS_NAME){var b=this.buildWptNode(c);
this.appendAttributesNode(b,a);return b}b=this.createElementNSPlus("gpx:trk");this.appendAttributesNode(b,a);a=this.buildTrkSegNode(c);a=OpenLayers.Util.isArray(a)?a:[a];for(var c=0,d=a.length;c<d;c++)b.appendChild(a[c]);return b},buildTrkSegNode:function(a){var c,b,d,e;if("OpenLayers.Geometry.LineString"==a.CLASS_NAME||"OpenLayers.Geometry.LinearRing"==a.CLASS_NAME){c=this.createElementNSPlus("gpx:trkseg");b=0;for(d=a.components.length;b<d;b++)e=a.components[b],c.appendChild(this.buildTrkPtNode(e));
return c}c=[];b=0;for(d=a.components.length;b<d;b++)c.push(this.buildTrkSegNode(a.components[b]));return c},buildTrkPtNode:function(a){var c=this.createElementNSPlus("gpx:trkpt");c.setAttribute("lon",a.x);c.setAttribute("lat",a.y);return c},buildWptNode:function(a){var c=this.createElementNSPlus("gpx:wpt");c.setAttribute("lon",a.x);c.setAttribute("lat",a.y);return c},appendAttributesNode:function(a,c){var b=this.createElementNSPlus("gpx:name");b.appendChild(this.createTextNode(c.attributes.name||
c.id));a.appendChild(b);b=this.createElementNSPlus("gpx:desc");b.appendChild(this.createTextNode(c.attributes.description||this.defaultDesc));a.appendChild(b)},CLASS_NAME:"OpenLayers.Format.GPX"});

//OpenLayers.Format.KML 2.12
OpenLayers.Format.KML=OpenLayers.Class(OpenLayers.Format.XML,{namespaces:{kml:"http://www.opengis.net/kml/2.2",gx:"http://www.google.com/kml/ext/2.2"},kmlns:"http://earth.google.com/kml/2.0",placemarksDesc:"No description available",foldersName:"OpenLayers export",foldersDesc:"Exported on "+new Date,extractAttributes:!0,kvpAttributes:!1,extractStyles:!1,extractTracks:!1,trackAttributes:null,internalns:null,features:null,styles:null,styleBaseUrl:"",fetched:null,maxDepth:0,initialize:function(a){this.regExes=
{trimSpace:/^\s*|\s*$/g,removeSpace:/\s*/g,splitSpace:/\s+/,trimComma:/\s*,\s*/g,kmlColor:/(\w{2})(\w{2})(\w{2})(\w{2})/,kmlIconPalette:/root:\/\/icons\/palette-(\d+)(\.\w+)/,straightBracket:/\$\[(.*?)\]/g};this.externalProjection=new OpenLayers.Projection("EPSG:4326");OpenLayers.Format.XML.prototype.initialize.apply(this,[a])},read:function(a){this.features=[];this.styles={};this.fetched={};return this.parseData(a,{depth:0,styleBaseUrl:this.styleBaseUrl})},parseData:function(a,c){"string"==typeof a&&
(a=OpenLayers.Format.XML.prototype.read.apply(this,[a]));for(var b=["Link","NetworkLink","Style","StyleMap","Placemark"],d=0,e=b.length;d<e;++d){var f=b[d],h=this.getElementsByTagNameNS(a,"*",f);if(0!=h.length)switch(f.toLowerCase()){case "link":case "networklink":this.parseLinks(h,c);break;case "style":this.extractStyles&&this.parseStyles(h,c);break;case "stylemap":this.extractStyles&&this.parseStyleMaps(h,c);break;case "placemark":this.parseFeatures(h,c)}}return this.features},parseLinks:function(a,
c){if(c.depth>=this.maxDepth)return!1;var b=OpenLayers.Util.extend({},c);b.depth++;for(var d=0,e=a.length;d<e;d++){var f=this.parseProperty(a[d],"*","href");f&&!this.fetched[f]&&(this.fetched[f]=!0,(f=this.fetchLink(f))&&this.parseData(f,b))}},fetchLink:function(a){if(a=OpenLayers.Request.GET({url:a,async:!1}))return a.responseText},parseStyles:function(a,c){for(var b=0,d=a.length;b<d;b++){var e=this.parseStyle(a[b]);e&&(this.styles[(c.styleBaseUrl||"")+"#"+e.id]=e)}},parseKmlColor:function(a){var c=
null;a&&(a=a.match(this.regExes.kmlColor))&&(c={color:"#"+a[4]+a[3]+a[2],opacity:parseInt(a[1],16)/255});return c},parseStyle:function(a){for(var c={},b=["LineStyle","PolyStyle","IconStyle","BalloonStyle","LabelStyle"],d,e,f=0,h=b.length;f<h;++f)if(d=b[f],e=this.getElementsByTagNameNS(a,"*",d)[0])switch(d.toLowerCase()){case "linestyle":d=this.parseProperty(e,"*","color");if(d=this.parseKmlColor(d))c.strokeColor=d.color,c.strokeOpacity=d.opacity;(d=this.parseProperty(e,"*","width"))&&(c.strokeWidth=
d);break;case "polystyle":d=this.parseProperty(e,"*","color");if(d=this.parseKmlColor(d))c.fillOpacity=d.opacity,c.fillColor=d.color;"0"==this.parseProperty(e,"*","fill")&&(c.fillColor="none");"0"==this.parseProperty(e,"*","outline")&&(c.strokeWidth="0");break;case "iconstyle":var g=parseFloat(this.parseProperty(e,"*","scale")||1);d=32*g;var j=32*g,k=this.getElementsByTagNameNS(e,"*","Icon")[0];if(k){var m=this.parseProperty(k,"*","href");if(m){var l=this.parseProperty(k,"*","w"),n=this.parseProperty(k,
"*","h");OpenLayers.String.startsWith(m,"http://maps.google.com/mapfiles/kml")&&(!l&&!n)&&(n=l=64,g/=2);l=l||n;n=n||l;l&&(d=parseInt(l)*g);n&&(j=parseInt(n)*g);if(n=m.match(this.regExes.kmlIconPalette))l=n[1],n=n[2],m=this.parseProperty(k,"*","x"),k=this.parseProperty(k,"*","y"),m="http://maps.google.com/mapfiles/kml/pal"+l+"/icon"+(8*(k?7-k/32:7)+(m?m/32:0))+n;c.graphicOpacity=1;c.externalGraphic=m}}if(e=this.getElementsByTagNameNS(e,"*","hotSpot")[0])m=parseFloat(e.getAttribute("x")),k=parseFloat(e.getAttribute("y")),
l=e.getAttribute("xunits"),"pixels"==l?c.graphicXOffset=-m*g:"insetPixels"==l?c.graphicXOffset=-d+m*g:"fraction"==l&&(c.graphicXOffset=-d*m),e=e.getAttribute("yunits"),"pixels"==e?c.graphicYOffset=-j+k*g+1:"insetPixels"==e?c.graphicYOffset=-(k*g)+1:"fraction"==e&&(c.graphicYOffset=-j*(1-k)+1);c.graphicWidth=d;c.graphicHeight=j;break;case "balloonstyle":(e=OpenLayers.Util.getXmlNodeValue(e))&&(c.balloonStyle=e.replace(this.regExes.straightBracket,"${$1}"));break;case "labelstyle":if(d=this.parseProperty(e,
"*","color"),d=this.parseKmlColor(d))c.fontColor=d.color,c.fontOpacity=d.opacity}!c.strokeColor&&c.fillColor&&(c.strokeColor=c.fillColor);if((a=a.getAttribute("id"))&&c)c.id=a;return c},parseStyleMaps:function(a,c){for(var b=0,d=a.length;b<d;b++)for(var e=a[b],f=this.getElementsByTagNameNS(e,"*","Pair"),e=e.getAttribute("id"),h=0,g=f.length;h<g;h++){var j=f[h],k=this.parseProperty(j,"*","key");(j=this.parseProperty(j,"*","styleUrl"))&&"normal"==k&&(this.styles[(c.styleBaseUrl||"")+"#"+e]=this.styles[(c.styleBaseUrl||
"")+j])}},parseFeatures:function(a,c){for(var b=[],d=0,e=a.length;d<e;d++){var f=a[d],h=this.parseFeature.apply(this,[f]);if(h){this.extractStyles&&(h.attributes&&h.attributes.styleUrl)&&(h.style=this.getStyle(h.attributes.styleUrl,c));if(this.extractStyles){var g=this.getElementsByTagNameNS(f,"*","Style")[0];if(g&&(g=this.parseStyle(g)))h.style=OpenLayers.Util.extend(h.style,g)}if(this.extractTracks){if((f=this.getElementsByTagNameNS(f,this.namespaces.gx,"Track"))&&0<f.length)h={features:[],feature:h},
this.readNode(f[0],h),0<h.features.length&&b.push.apply(b,h.features)}else b.push(h)}else throw"Bad Placemark: "+d;}this.features=this.features.concat(b)},readers:{kml:{when:function(a,c){c.whens.push(OpenLayers.Date.parse(this.getChildValue(a)))},_trackPointAttribute:function(a,c){var b=a.nodeName.split(":").pop();c.attributes[b].push(this.getChildValue(a))}},gx:{Track:function(a,c){var b={whens:[],points:[],angles:[]};if(this.trackAttributes){var d;b.attributes={};for(var e=0,f=this.trackAttributes.length;e<
f;++e)d=this.trackAttributes[e],b.attributes[d]=[],d in this.readers.kml||(this.readers.kml[d]=this.readers.kml._trackPointAttribute)}this.readChildNodes(a,b);if(b.whens.length!==b.points.length)throw Error("gx:Track with unequal number of when ("+b.whens.length+") and gx:coord ("+b.points.length+") elements.");var h=0<b.angles.length;if(h&&b.whens.length!==b.angles.length)throw Error("gx:Track with unequal number of when ("+b.whens.length+") and gx:angles ("+b.angles.length+") elements.");for(var g,
j,e=0,f=b.whens.length;e<f;++e){g=c.feature.clone();g.fid=c.feature.fid||c.feature.id;j=b.points[e];g.geometry=j;"z"in j&&(g.attributes.altitude=j.z);this.internalProjection&&this.externalProjection&&g.geometry.transform(this.externalProjection,this.internalProjection);if(this.trackAttributes){j=0;for(var k=this.trackAttributes.length;j<k;++j)g.attributes[d]=b.attributes[this.trackAttributes[j]][e]}g.attributes.when=b.whens[e];g.attributes.trackId=c.feature.id;h&&(j=b.angles[e],g.attributes.heading=
parseFloat(j[0]),g.attributes.tilt=parseFloat(j[1]),g.attributes.roll=parseFloat(j[2]));c.features.push(g)}},coord:function(a,c){var b=this.getChildValue(a).replace(this.regExes.trimSpace,"").split(/\s+/),d=new OpenLayers.Geometry.Point(b[0],b[1]);2<b.length&&(d.z=parseFloat(b[2]));c.points.push(d)},angles:function(a,c){var b=this.getChildValue(a).replace(this.regExes.trimSpace,"").split(/\s+/);c.angles.push(b)}}},parseFeature:function(a){for(var c=["MultiGeometry","Polygon","LineString","Point"],
b,d,e,f=0,h=c.length;f<h;++f)if(b=c[f],this.internalns=a.namespaceURI?a.namespaceURI:this.kmlns,d=this.getElementsByTagNameNS(a,this.internalns,b),0<d.length){if(c=this.parseGeometry[b.toLowerCase()])e=c.apply(this,[d[0]]),this.internalProjection&&this.externalProjection&&e.transform(this.externalProjection,this.internalProjection);else throw new TypeError("Unsupported geometry type: "+b);break}var g;this.extractAttributes&&(g=this.parseAttributes(a));b=new OpenLayers.Feature.Vector(e,g);a=a.getAttribute("id")||
a.getAttribute("name");null!=a&&(b.fid=a);return b},getStyle:function(a,c){var b=OpenLayers.Util.removeTail(a),d=OpenLayers.Util.extend({},c);d.depth++;d.styleBaseUrl=b;!this.styles[a]&&!OpenLayers.String.startsWith(a,"#")&&d.depth<=this.maxDepth&&!this.fetched[b]&&(b=this.fetchLink(b))&&this.parseData(b,d);return OpenLayers.Util.extend({},this.styles[a])},parseGeometry:{point:function(a){var c=this.getElementsByTagNameNS(a,this.internalns,"coordinates");a=[];if(0<c.length){var b=c[0].firstChild.nodeValue,
b=b.replace(this.regExes.removeSpace,"");a=b.split(",")}c=null;if(1<a.length)2==a.length&&(a[2]=null),c=new OpenLayers.Geometry.Point(a[0],a[1],a[2]);else throw"Bad coordinate string: "+b;return c},linestring:function(a,c){var b=this.getElementsByTagNameNS(a,this.internalns,"coordinates"),d=null;if(0<b.length){for(var b=this.getChildValue(b[0]),b=b.replace(this.regExes.trimSpace,""),b=b.replace(this.regExes.trimComma,","),d=b.split(this.regExes.splitSpace),e=d.length,f=Array(e),h,g,j=0;j<e;++j)if(h=
d[j].split(","),g=h.length,1<g)2==h.length&&(h[2]=null),f[j]=new OpenLayers.Geometry.Point(h[0],h[1],h[2]);else throw"Bad LineString point coordinates: "+d[j];if(e)d=c?new OpenLayers.Geometry.LinearRing(f):new OpenLayers.Geometry.LineString(f);else throw"Bad LineString coordinates: "+b;}return d},polygon:function(a){a=this.getElementsByTagNameNS(a,this.internalns,"LinearRing");var c=a.length,b=Array(c);if(0<c)for(var d=0,e=a.length;d<e;++d)if(c=this.parseGeometry.linestring.apply(this,[a[d],!0]))b[d]=
c;else throw"Bad LinearRing geometry: "+d;return new OpenLayers.Geometry.Polygon(b)},multigeometry:function(a){for(var c,b=[],d=a.childNodes,e=0,f=d.length;e<f;++e)a=d[e],1==a.nodeType&&(c=a.prefix?a.nodeName.split(":")[1]:a.nodeName,(c=this.parseGeometry[c.toLowerCase()])&&b.push(c.apply(this,[a])));return new OpenLayers.Geometry.Collection(b)}},parseAttributes:function(a){var c={},b=a.getElementsByTagName("ExtendedData");b.length&&(c=this.parseExtendedData(b[0]));var d,e,f;a=a.childNodes;for(var b=
0,h=a.length;b<h;++b)if(d=a[b],1==d.nodeType&&(e=d.childNodes,1<=e.length&&3>=e.length)){switch(e.length){case 1:f=e[0];break;case 2:f=e[0];e=e[1];f=3==f.nodeType||4==f.nodeType?f:e;break;default:f=e[1]}if(3==f.nodeType||4==f.nodeType)if(d=d.prefix?d.nodeName.split(":")[1]:d.nodeName,f=OpenLayers.Util.getXmlNodeValue(f))f=f.replace(this.regExes.trimSpace,""),c[d]=f}return c},parseExtendedData:function(a){var c={},b,d,e,f,h=a.getElementsByTagName("Data");b=0;for(d=h.length;b<d;b++){e=h[b];f=e.getAttribute("name");
var g={},j=e.getElementsByTagName("value");j.length&&(g.value=this.getChildValue(j[0]));this.kvpAttributes?c[f]=g.value:(e=e.getElementsByTagName("displayName"),e.length&&(g.displayName=this.getChildValue(e[0])),c[f]=g)}a=a.getElementsByTagName("SimpleData");b=0;for(d=a.length;b<d;b++)g={},e=a[b],f=e.getAttribute("name"),g.value=this.getChildValue(e),this.kvpAttributes?c[f]=g.value:(g.displayName=f,c[f]=g);return c},parseProperty:function(a,c,b){var d;a=this.getElementsByTagNameNS(a,c,b);try{d=OpenLayers.Util.getXmlNodeValue(a[0])}catch(e){d=
null}return d},write:function(a){OpenLayers.Util.isArray(a)||(a=[a]);for(var c=this.createElementNS(this.kmlns,"kml"),b=this.createFolderXML(),d=0,e=a.length;d<e;++d)b.appendChild(this.createPlacemarkXML(a[d]));c.appendChild(b);return OpenLayers.Format.XML.prototype.write.apply(this,[c])},createFolderXML:function(){var a=this.createElementNS(this.kmlns,"Folder");if(this.foldersName){var c=this.createElementNS(this.kmlns,"name"),b=this.createTextNode(this.foldersName);c.appendChild(b);a.appendChild(c)}this.foldersDesc&&
(c=this.createElementNS(this.kmlns,"description"),b=this.createTextNode(this.foldersDesc),c.appendChild(b),a.appendChild(c));return a},createPlacemarkXML:function(a){var c=this.createElementNS(this.kmlns,"name");c.appendChild(this.createTextNode(a.style&&a.style.label?a.style.label:a.attributes.name||a.id));var b=this.createElementNS(this.kmlns,"description");b.appendChild(this.createTextNode(a.attributes.description||this.placemarksDesc));var d=this.createElementNS(this.kmlns,"Placemark");null!=
a.fid&&d.setAttribute("id",a.fid);d.appendChild(c);d.appendChild(b);c=this.buildGeometryNode(a.geometry);d.appendChild(c);a.attributes&&(a=this.buildExtendedData(a.attributes))&&d.appendChild(a);return d},buildGeometryNode:function(a){var c=a.CLASS_NAME,c=c.substring(c.lastIndexOf(".")+1),c=this.buildGeometry[c.toLowerCase()],b=null;c&&(b=c.apply(this,[a]));return b},buildGeometry:{point:function(a){var c=this.createElementNS(this.kmlns,"Point");c.appendChild(this.buildCoordinatesNode(a));return c},
multipoint:function(a){return this.buildGeometry.collection.apply(this,[a])},linestring:function(a){var c=this.createElementNS(this.kmlns,"LineString");c.appendChild(this.buildCoordinatesNode(a));return c},multilinestring:function(a){return this.buildGeometry.collection.apply(this,[a])},linearring:function(a){var c=this.createElementNS(this.kmlns,"LinearRing");c.appendChild(this.buildCoordinatesNode(a));return c},polygon:function(a){var c=this.createElementNS(this.kmlns,"Polygon");a=a.components;
for(var b,d,e=0,f=a.length;e<f;++e)b=0==e?"outerBoundaryIs":"innerBoundaryIs",b=this.createElementNS(this.kmlns,b),d=this.buildGeometry.linearring.apply(this,[a[e]]),b.appendChild(d),c.appendChild(b);return c},multipolygon:function(a){return this.buildGeometry.collection.apply(this,[a])},collection:function(a){for(var c=this.createElementNS(this.kmlns,"MultiGeometry"),b,d=0,e=a.components.length;d<e;++d)(b=this.buildGeometryNode.apply(this,[a.components[d]]))&&c.appendChild(b);return c}},buildCoordinatesNode:function(a){var c=
this.createElementNS(this.kmlns,"coordinates"),b;if(b=a.components){for(var d=b.length,e=Array(d),f=0;f<d;++f)a=b[f],e[f]=this.buildCoordinates(a);b=e.join(" ")}else b=this.buildCoordinates(a);b=this.createTextNode(b);c.appendChild(b);return c},buildCoordinates:function(a){this.internalProjection&&this.externalProjection&&(a=a.clone(),a.transform(this.internalProjection,this.externalProjection));return a.x+","+a.y},buildExtendedData:function(a){var c=this.createElementNS(this.kmlns,"ExtendedData"),
b;for(b in a)if(a[b]&&"name"!=b&&"description"!=b&&"styleUrl"!=b){var d=this.createElementNS(this.kmlns,"Data");d.setAttribute("name",b);var e=this.createElementNS(this.kmlns,"value");if("object"==typeof a[b]){if(a[b].value&&e.appendChild(this.createTextNode(a[b].value)),a[b].displayName){var f=this.createElementNS(this.kmlns,"displayName");f.appendChild(this.getXMLDoc().createCDATASection(a[b].displayName));d.appendChild(f)}}else e.appendChild(this.createTextNode(a[b]));d.appendChild(e);c.appendChild(d)}return this.isSimpleContent(c)?
null:c},CLASS_NAME:"OpenLayers.Format.KML"}); 