/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Historydata.js 3320 2013-02-26 14:22:17Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Historydata
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Historydata = OpenLayers.Class(Giscuit.Control, {

    downloadControl: null,

    panel: null,

    downloadGeometry: null,
    
    uploadErrorDt: null,
    activateDs: null,
    activateDt: null,
    activateDtHeightCoef: 60,
    reportDts: [],

    selected: null,
    
    submitButton: null,
    submitDownloadButton: null,
    activateSubmitButton: null,
    reportButton: null,
    activateDeleteButton: null,
    activateModeButton: null,
    reportBackButton: null,
    
    uploadTooltipShown: false,
    downloadTooltipShown: false,
            
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);

        if (deactivated == true) {
            if(this.downloadControl != null) {
                this.downloadControl.destroy();
                this.downloadControl = null;
            }
            
            //Uncheck button
            YAHOO.util.Dom.removeClass(
                gt.control.editgeom.button.get('element'), 
                'yui-checkbox-button-checked'
            );
        }
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },

    clear: function() {
        this.deactivate();
        
        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        var elements = [
           'activateDt' ,
           'uploadErrorDt' ,    
           'submitButton' ,
           'submitDownloadButton' ,
           'activateSubmitButton' ,
           'reportButton' ,
           'activateDeleteButton' ,
           'reportBackButton'
        ];
        for(var i=0; i < elements.length; i++) {
            if(this[elements[i]] != null) {
                this[elements[i]].destroy();
                this[elements[i]] = null;
            }
        }
        
        //Destroy report datatables
        if(this.reportDts.length) {
            for(var i=0; i < this.reportDts.length; i++) {
                this.reportDts[i].destroy();
            }
            this.reportDts = [];
        }
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    /**
     * Upload
     */
    initUploadInput: function () {
        if(YAHOO.util.Dom.get('gtHistorydataUploadSubmit') != null) {
            this.submitButton = new YAHOO.widget.Button('gtHistorydataUploadSubmit');
            this.submitButton.on("click", this.submit, null, this);
        }
    },
    
    uploadActivate: function () {
        this.clear();
        
        //Check button
        gt.control.editgeom.button.set("label", Giscuit.Util.getMenuItemById(
            gt.control.editgeom.button, 
            'gtDatamanageUploadControl'
        ).cfg.getProperty('text'));
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('datamanage') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#offline.upload');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());
        
        var getCallback = {
            success: this.uploadgetSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/uploadget", getCallback);
    },
    
    uploadgetSuccess: function (o) {
        gt.accord.setBody(this.getPanelIndex(), o.responseText);
        this.initUploadInput();
    },
    
    uploadDone: function (o) {
        if (o.responseXML && o.responseXML.body) {
            var responseText = o.responseXML.body.innerHTML;
        } else {
            var responseText = o.responseText;
        }
        gt.accord.setBody(this.getPanelIndex(), responseText);
        
        var errorData = YAHOO.util.Dom.get('gtUploadErrorData');

        var optionsArr = [
            {label:gt.translate._("leaveUnresolved"), value: 0}, 
            {label:gt.translate._("discardMyVersion"), value: 1}, 
            {label:gt.translate._("commitMyVersion"), value: 2}
        ];
        
        var actionFormatter = function (elCell, oRecord, oColumn, oData) {
            var value = oRecord.getData(oColumn.key);
            
            if(value == '' || value == 0) {
                elCell.innerHTML = gt.translate._('leaveUnresolved');
            } else if (value == 1){
                elCell.innerHTML = gt.translate._('discardMyVersion');
            } else if (value == 2) {
                elCell.innerHTML = gt.translate._('commitMyVersion');
            }
        };
        
        if (errorData != null) {
            var myColumnDefs = [{
                key: "layer",
                label: gt.translate._("layer"),
                sortable: true
            },
            {
                key: "index",
                label: gt.translate._("index"),
                sortable: true
            },
            {
                key: "field",
                label: gt.translate._("field"),
                sortable: true
            },
            {
                key: "error",
                label: gt.translate._("error"),
                sortable: true
            },
            {
                key: "action",
                label: gt.translate._("action"),
                editor: new YAHOO.widget.RadioCellEditor({radioOptions: optionsArr, disableBtns:true}),
                formatter: actionFormatter
            },
            {
                key: "gid",
                label: gt.translate._("gid"),
                hidden: true
            },
            {
                key: "geometry",
                label: gt.translate._("geometry"),
                hidden: true
            }
            ];
            
            var myDataSource = new YAHOO.util.DataSource(errorData);
            myDataSource.responseType = YAHOO.util.DataSource.TYPE_HTMLTABLE;
            myDataSource.responseSchema = {
                fields: ["layer", "index", "field", "error", "action", "gid", "geometry"]
            };

            this.uploadErrorDt = new YAHOO.widget.DataTable("gtUploadErrorTable", myColumnDefs, myDataSource);
            this.uploadErrorDt.subscribe("cellClickEvent", function (oArgs) {
                var column = this.uploadErrorDt.getColumn(oArgs.target).key;

                if (column != 'action') {
                    this.uploadErrorDt.onEventSelectRow(oArgs);
                    
                    var row = this.uploadErrorDt.getSelectedRows();
                    var geometry = OpenLayers.Geometry.fromWKT(this.uploadErrorDt.getRecord(row[0]).getData('geometry'));

                    if(this.selected != null) {
                        gt.layer.vector.destroyFeatures([this.selected]);
                        this.selected = null;
                    }
                    
                    this.selected = new OpenLayers.Feature.Vector(geometry);
                    gt.layer.vector.addFeatures([this.selected]);
                    gt.map.zoomToExtent(geometry.getBounds());
                    
                } else {
                    this.uploadErrorDt.onEventShowCellEditor(oArgs);
                }
            }, null, this);
            
            var reuploadButton = new YAHOO.widget.Button('gtUploadReupload');
            reuploadButton.on("click", this.reupload, null, this);
            
            //
            if (! this.uploadTooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('conflictResolveHelp'));
                gt.tooltip.showTooltip();
                this.uploadTooltipShown = true;
            }
            
        }

        this.initUploadInput();
    },
    
    reupload: function (evt) {
        var skipArr = [];
        var overwriteArr = [];

        var records = this.uploadErrorDt.getRecordSet().getRecords();
        for (var i = 0; i < records.length; i++) {
            var data = records[i].getData();
            if (typeof data.action != 'undefined' && data.action == 1) {
                skipArr.push(data.gid);
            } else if (typeof data.action != 'undefined' && data.action == 2) {
                overwriteArr.push(data.gid);
            }
        }
        
        var filename = YAHOO.util.Dom.get('gtUploadFilename').value;

        var url = gt.config.httpToolPath + '/historydata/upload/filename/' + filename;
        if (skipArr.length != 0) {
            url += '/skip/' + skipArr.join();
        }
        if (overwriteArr.length != 0) {
            url += '/overwrite/' + overwriteArr.join();
        }

        gt.accord.setBody(this.getPanelIndex(), '<img id="gtHistorydataUploadLoader" src="' + gt.config.httpImagePath + '/gt-ajax-loader.gif">');               

        var uploadCallback = {
            success: this.uploadDone, 
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.asyncRequest('POST', url, uploadCallback);
    },
    
    submit: function () {
        var uploadCallback = {
            upload: this.uploadDone,
            failure: this.callbackFailure,
            scope: this
        };

        //Show loader
        YAHOO.util.Dom.get('gtHistorydataUploadLoader').style.display = "block";
        YAHOO.util.Dom.get('gtHistorydataUploadForm').style.display = "none";
        
        YAHOO.util.Connect.setForm('gtHistorydataUploadForm', true);
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/historydata/upload", uploadCallback);
    },

    /**
     * Download 
     */
    downloadActivate: function () {
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated == true) {
            
            //Tooltip
            if (! this.downloadTooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('drawGeometryHelp'));
                gt.tooltip.showTooltip();
                this.downloadTooltipShown = true;
            }

            this.downloadControl = new OpenLayers.Control.DrawFeature(
            gt.layer.vector, OpenLayers.Handler.Polygon, {
                callbacks: {
                    "done": OpenLayers.Function.bind(this.downloadDone, this)
                }
            });

            gt.map.addControl(this.downloadControl);
            this.downloadControl.activate();
            
            //Check button
            gt.control.editgeom.button.set("label", Giscuit.Util.getMenuItemById(
                gt.control.editgeom.button, 
                'gtDatamanageDownloadControl'
            ).cfg.getProperty('text'));
            YAHOO.util.Dom.addClass(
                gt.control.editgeom.button.get('element'), 
                'yui-checkbox-button-checked'
            );
        }

    },

    downloadSubmit: function () {
        this.submitDownloadButton.set('disabled', true);

        var submitCallback = {
            success: this.downloadSubmitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.setForm('gtHistorydataDownloadForm');
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/historydata/downloadcreate/geometry/" + this.downloadGeometry, submitCallback);
    },

    downloadSubmitSuccess: function (o) {

        var responseText = YAHOO.lang.JSON.parse(o.responseText);

        //Error in form
        if (responseText.form != undefined) {
            gt.accord.setBody(this.getPanelIndex(), responseText.form);
            this.initDownloadInput();
        }

        //Download file
        if (responseText.file) {
            var url = gt.config.httpToolPath + '/historydata/download/file/' + responseText.file;
            if (YAHOO.env.ua.ie) {
                var html = gt.translate._('fileCreatedIe').replace('%1$s', url);
                gt.accord.setBody(this.getPanelIndex(), html);
            } else {
                var html = '<p>' + gt.translate._('fileCreated') + '</p>';
                gt.accord.setBody(this.getPanelIndex(), html);
                window.location = url;
            }
        } else {
            if (responseText.msg != undefined) {
                gt.accord.setBody(this.getPanelIndex(), responseText.msg);
            }
        }

        this.submitDownloadButton.set('disabled', false);
    },

    initDownloadInput: function () {
        this.submitDownloadButton = new YAHOO.widget.Button('gtHistorydataDownloadSubmit');
        this.submitDownloadButton.on("click", this.downloadSubmit, null, this);
    },

    downloadDone: function (geometry) {
        this.downloadGeometry = geometry;

        this.downloadControl.deactivate();
        this.clear();
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('datamanage') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#offline.download');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());

        var getCallback = {
            success: this.downloadSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/downloadget", getCallback);

    },

    downloadSuccess: function (o) {

        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }

        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }

        if (typeof(responseText.form) != 'undefined') {
            gt.accord.setBody(this.getPanelIndex(), responseText.form);
            this.initDownloadInput();
        }

    },

    /**
     * Activate
     */
    activateActivate: function () {
        this.clear();
        
        //Check button
        gt.control.editgeom.button.set("label", Giscuit.Util.getMenuItemById(
            gt.control.editgeom.button, 
            'gtDatamanageActivateControl'
        ).cfg.getProperty('text'));
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('datamanage') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#offline.activate');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.openPanel(this.getPanelIndex());
        
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);
        
        var getCallback = {
            success: this.activategetSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/activate", getCallback);
    },

    createActivateDataTable: function (o) {
        gt.accord.setBody(this.getPanelIndex(), o.responseText);

        this.initActivateInput();

        //Define columns
        var columnDefs = [{
            key: "id",
            resizeable: true,
            sortable: true,
            label: gt.translate._('id')
        } , 
        {
            key: "user_name",
            resizeable: true,
            sortable: true,
            label: gt.translate._('user_name')
        } ,
        {
            key: "upload_date",
            resizeable: true,
            sortable: true,
            label: gt.translate._('date')
        }];

        //Create data source
        this.activateDs = new YAHOO.util.DataSource(gt.config.httpToolPath + "/historydata/activateget");
        this.activateDs.responseType = YAHOO.util.DataSource.TYPE_JSON;
        this.activateDs.responseSchema = {
            resultsList: "records",
            fields: ["id", "user_name", "upload_date", 'upload_extent']
        };

        //Create data table
        this.activateDt = new YAHOO.widget.DataTable("gtUploadActivateDt", columnDefs, this.activateDs, {
            scrollable: true,
            width: '99.5%',
            selectionMode: "single",
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });

        this.setOptimalDatatableHeight(this.activateDt, this.activateDtHeightCoef);
        
        this.activateDt.subscribe("rowMouseoverEvent", this.activateDt.onEventHighlightRow);
        this.activateDt.subscribe("rowMouseoutEvent", this.activateDt.onEventUnhighlightRow);

        //Select row
        this.activateDt.subscribe("cellClickEvent", function (oArgs) {

            this.activateDt.onEventSelectRow(oArgs);

            this.reportButton.set('disabled', false);
            this.activateSubmitButton.set('disabled', false);
            this.activateDeleteButton.set('disabled', false);

            //Zoom to extent
            var row = this.activateDt.getSelectedRows();
            
            var uploadExtent = this.activateDt.getRecord(row[0]).getData('upload_extent');
            uploadExtent = uploadExtent.substr(4, uploadExtent.length - 5).replace(/ /g, ",");
            uploadExtent = OpenLayers.Bounds.fromString(uploadExtent);
            gt.map.zoomToExtent(uploadExtent);

        }, null, this);

    },

    activategetSuccess: function (o) {
        this.createActivateDataTable(o);
    },

    initActivateInput: function () {

        this.activateSubmitButton = new YAHOO.widget.Button('gtUploadActivateSubmit');
        this.activateSubmitButton.set('disabled', true);
        this.activateSubmitButton.on("click", this.activateSubmit, null, this);

        this.reportButton = new YAHOO.widget.Button('gtUploadReport');
        this.reportButton.set('disabled', true);
        this.reportButton.on("click", this.report, null, this);
        
        this.activateDeleteButton = new YAHOO.widget.Button('gtUploadActivateDelete');
        this.activateDeleteButton.set('disabled', true);
        this.activateDeleteButton.on("click", this.activateDelete, null, this);
        
        this.reportBackButton = new YAHOO.widget.Button('gtUploadReportBack');
        this.reportBackButton.on("click", this.reportBack, null, this);
        YAHOO.util.Dom.get('gtUploadReportBack').style.display = 'none';
        
        /*
        this.activateModeButton = YAHOO.util.Dom.get('gtUploadActivateMode');
        YAHOO.util.Event.addListener(this.activateModeButton, "change", this.activateMode, null, this);
        */
    },

    activateModeSuccess: function (o) {
        this.activateModeButton.disabled = false;
        gt.map.refresh();
    },

    /*
    activateMode: function (e) {
        
        if(e.target.checked == true) {
            var value = 't';
        } else {
            var value = 'f';
        }
        
        this.activateModeButton.disabled = true;

        var modeCallback = {
            success: this.activateModeSuccess,
            scope: this,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/activatemode/value/" + value, modeCallback);
    },
    */

    activateDelete: function () {
        var answer = confirm(gt.translate._('deleteConfirm'));
        if (! answer) {
            return;
        }

        var selected = this.activateDt.getSelectedRows();
        var record = this.activateDt.getRecord(selected[0]);

        //Get id
        var id = record.getData('id');

        var submitCallback = {
            success: this.activateSubmitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/activatedelete/id/" + id, submitCallback);

        //Delete row
        this.activateDt.deleteRow(record);

        gt.map.refresh();
    },
    
    activateSubmit: function () {

        var selected = this.activateDt.getSelectedRows();
        var record = this.activateDt.getRecord(selected[0]);

        //Get id
        var id = record.getData('id');

        var submitCallback = {
            success: this.activateSubmitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/activatesubmit/id/" + id, submitCallback);

        //Delete row
        this.activateDt.deleteRow(record);

        gt.map.refresh();
    },

    activateSubmitSuccess: function () {
        
    },

    /**
     * Report
     */
    reportBack: function() {
        this.reportBackButton.set('disabled', true);
        
        var getCallback = {
            success: this.reportBackSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/reportback", getCallback);
    },
    
    reportBackSuccess: function (o) {
        gt.map.refresh();
        
        this.reportBackButton.set('disabled', false);
        
        YAHOO.util.Dom.get('gtUploadActivateInput').style.display = 'block';
        YAHOO.util.Dom.get('gtUploadActivateDt').style.display = 'block';
        YAHOO.util.Dom.get('gtUploadReportDtPH').style.display = 'none';
        YAHOO.util.Dom.get('gtUploadReportBack').style.display = 'none';
    },
    
    report: function () {

        var selected = this.activateDt.getSelectedRows();
        var id = this.activateDt.getRecord(selected[0]).getData('id');

        YAHOO.util.Dom.get('gtUploadActivateInput').style.display = 'none';
        YAHOO.util.Dom.get('gtUploadActivateDt').style.display = 'none';
        YAHOO.util.Dom.get('gtUploadReportDtPH').style.display = 'block';
        YAHOO.util.Dom.get('gtUploadReportBack').style.display = 'inline-block';

        var getCallback = {
            success: this.reportSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historydata/report/id/" + id, getCallback);

    },

    reportSuccess: function (o) {
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        //FIXME:
        /*
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        */
        
        gt.map.refresh();
        
        //Destroy report datatables
        if(this.reportDts.length) {
            for(var i=0; i < this.reportDts.length; i++) {
                this.reportDts[i].destroy();
            }
            this.reportDts = [];
        }
        
        YAHOO.util.Dom.get('gtUploadReportDtPH').innerHTML = '';

        for (var tableName in responseText) {
            this.createReportTable(responseText[tableName], tableName);
        }
    },

    createReportTable: function (data, tableName) {

        //Create data source
        var ds = new YAHOO.util.DataSource(data['records']);
        ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        ds.responseSchema = {
            fields: data['fields']
        };
        
        //
        var brEl = document.createElement("br");
        YAHOO.util.Dom.get('gtUploadReportDtPH').appendChild(brEl);
        
        //Create a placeholder for table name
        var textDiv = document.createElement("span");
        textDiv.id = tableName + "Text";
        textDiv.innerHTML = data['name'] + ':';
        YAHOO.util.Dom.get('gtUploadReportDtPH').appendChild(textDiv);
        
        //Create a placeholder for the data table
        var dtDiv = document.createElement("div");
        dtDiv.id = tableName + "Dt";
        YAHOO.util.Dom.get('gtUploadReportDtPH').appendChild(dtDiv);
        
        
        
        //Create data table
        var dt = new YAHOO.widget.DataTable(dtDiv.id, data['columns'], ds, {
            scrollable: true,
            width: '99.5%',
            height: '100px',
            selectionMode: 'single',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        //Select row
        dt.subscribe("cellClickEvent", function (oArgs) {
            dt.onEventSelectRow(oArgs);
            
            var row = dt.getSelectedRows();
            
            var geomtryWkt = dt.getRecord(row[0]).getData('the_geom');
                        
            if(geomtryWkt instanceof Array) {
                geomtryWkt = geomtryWkt[1];
            }
            
            var geometry = OpenLayers.Geometry.fromWKT(geomtryWkt);
            
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            this.selected = new OpenLayers.Feature.Vector(geometry);
            
            gt.layer.vector.addFeatures([this.selected]);
            
            gt.map.zoomToExtent(geometry.getBounds());
            
        }, null, this);
        
        this.reportDts.push(dt);
    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.activateDt) {
            this.setOptimalDatatableHeight(this.activateDt, this.activateDtHeightCoef);
        }
    },

    CLASS_NAME: "Giscuit.Control.Historydata"

});