/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Datatable.js 3449 2013-06-13 08:20:20Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Datatable
 * Contains functions for YUI datatable manipulation
 */ 
Giscuit.Datatable = function (options) {
    //
    this.currRecord = null;
    this.movetoRecordIndex = null;
    this.movetoRecord = null;
    this.dataUpdated = false;
    
    //Params
    if(options.initialLoad === undefined) options.initialLoad = true;
    if(options.scrollable === undefined) options.scrollable = false;
//    if(options.paginated === undefined) options.paginated = false;
    if(options.height === undefined) options.height = false;
    if(options.width === undefined) options.width = false;
    if(options.dataSourceType === undefined) options.dataSourceType = YAHOO.util.DataSource.TYPE_JSON;
    if(options.selectionMode === undefined) options.selectionMode = 'standard';
    if(options.checkCellUpdate === undefined) options.checkCellUpdate = false;
    if(options.paginatorAlwaysVisible === undefined) options.paginatorAlwaysVisible = true;
   
    //Events
    options.onCellClick = new YAHOO.util.CustomEvent("onCellClick", this); 
    options.onRowAdd = new YAHOO.util.CustomEvent("onRowAdd");
    options.afterRowDelete = new YAHOO.util.CustomEvent("afterRowDelete"); 
    options.orderDragEndEvent = new YAHOO.util.CustomEvent("orderDragEndEvent");
    
    //
    this.options = options;
};

Giscuit.Datatable.prototype.initialize = function () {
    //Create data source
    this.ds = new YAHOO.util.DataSource(this.options.dataUrl);
    this.ds.responseType = this.options.dataSourceType;
    this.ds.responseSchema = {
        resultsList: "records",
        fields: this.options.fields
    };

    //Paginator
    var paginator = null;
    if(this.options.rowsPerPage != undefined) {
        paginator = new YAHOO.widget.Paginator({
            alwaysVisible: this.options.paginatorAlwaysVisible,
            rowsPerPage: this.options.rowsPerPage,
            nextPageLinkLabel: gt.translate._('nextPage'),
            previousPageLinkLabel: gt.translate._('previousPage'),
            firstPageLinkLabel: gt.translate._('firstPage'),
            lastPageLinkLabel: gt.translate._('lastPage')
        });
//        this.options.paginated = true;
    }

    this.dt = new YAHOO.widget.DataTable(this.options.container, this.options.columnDefs, this.ds, {
        MSG_EMPTY: gt.translate._('emptyResult'),
        MSG_LOADING: gt.translate._('loading'),
        paginated: this.options.paginated,
        initialLoad: this.options.initialLoad,
        scrollable: this.options.scrollable,
        height: this.options.height,
        width: this.options.width,
        selectionMode: this.options.selectionMode,
        "paginator": paginator
    });
    
    //Order
    if(this.options.scrollable == true) {
        if(this.options.orderUrl) {
            this.dt.orderTargets = {};
            this.dt.orderDragEndEvent = this.options.orderDragEndEvent;
            
            this.dt.subscribe('cellMousedownEvent', this.orderCellMousedownEvent);
            this.dt.subscribe('initEvent', this.orderInitEvent);
            this.dt.subscribe('rowAddEvent', this.orderRowAddEvent);
            this.dt.orderDragEndEvent.subscribe(this.orderEnd, null, this);
        }
    }    
        
    if(this.options.selectionMode != undefined) {
        // Mouseover
        this.dt.subscribe("cellMouseoverEvent", function (oArgs) {
            var column = this.dt.getColumn(oArgs.target);
            var cell = this.dt.getCell(oArgs.target);
    
            if (column.key == this.options.keyColumn) {
                this.dt.highlightCell(cell);
            }
        }, null, this);

        // Mouseout
        this.dt.subscribe("cellMouseoutEvent", function (oArgs) {
            var column = this.dt.getColumn(oArgs.target);
            var cell = this.dt.getCell(oArgs.target);
    
            if (column.key == this.options.keyColumn) {
                this.dt.unhighlightCell(cell);
            }
        }, null, this);
    

        // Select row
          this.dt.subscribe("cellClickEvent", this.cellClick, null, this);
        this.dt.subscribe("rowUnselectEvent", this.unselectRows, null, this);
    }

    // Edit
    this.dt.subscribe("cellDblclickEvent", this.dt.onEventShowCellEditor);

    this.dt.subscribe("editorBlurEvent", function (oArgs) {
        this.cancelCellEditor();
    });

    this.dt.subscribe("editorSaveEvent", this.editRow, null, this);

    // Events
    if (this.deleteButton != undefined) {
        this.deleteButton.on("click", this.deleteRows, null, this);
    }

    if (this.addButton != undefined) {
        this.addButton.on("click", this.addRow, null, this);
    }
    
    if(this.moveupButton != undefined && this.movedownButton) {
        this.moveupButton.on("click", this.moveSelectedRow, -1, this);    
        this.movedownButton.on("click", this.moveSelectedRow, 1, this);
        
        this.options.onCellClick.subscribe(this.checkInputButtons, null, this);
    }
    
    if(this.options.checkCellUpdate == true) {
        this.dt.subscribe("rowAddEvent", this.dataChanged, null, this);
        this.dt.subscribe("cellUpdateEvent", this.dataChanged, null, this);
        this.dt.subscribe("rowDeleteEvent", this.dataChanged, null, this);
    }
    
};

Giscuit.Datatable.prototype.unselectRows = function (oArgs) {
    var selected = this.dt.getSelectedRows();
    
    if(selected.length != 0) {
        return;
    }
    
    if(this.deleteButton != undefined) {
        this.deleteButton.set('disabled', true);
    }
    
    if(this.editButton != undefined) {
        this.editButton.set('disabled', true);
    }
    
    if(this.movedownButton != undefined) {
        this.movedownButton.set('disabled', true);
    }
    
    if(this.movedownButton != undefined) {
        this.movedownButton.set('disabled', true);
    }
    
};

Giscuit.Datatable.prototype.cellClick = function (oArgs) {
    //Select row by name column
    var column = this.dt.getColumn(oArgs.target).key;
    if (column == this.options.keyColumn) {
        this.dt.onEventSelectRow(oArgs);
    }

    var row = this.dt.getSelectedRows();
    var id = this.dt.getRecord(row[0]).getData(this.keyColumn);

    if (this.deleteButton != undefined) {
        this.deleteButton.set('disabled', false);
    }
    
    if (this.editButton != undefined) {
        this.editButton.set('disabled', false);
    }

    this.options.onCellClick.fire(); 
};

//Delete
Giscuit.Datatable.prototype.deleteRows = function () {
    if (confirm(gt.translate._('deleteConfirm'))) {

        this.deleteButton.set('disabled', true);

        var rows = this.dt.getSelectedRows();
        var ids = new Array(rows.length);

        for (var i = 0; i < rows.length; i++) {
            ids[i] = this.dt.getRecord(rows[i]).getData(this.options.keyColumn);
        }

        for(var i = 0; i < rows.length; i++) {
            this.dt.deleteRow(rows[i]);
        }
        
        ids = ids.join(',');

        YAHOO.util.Connect.asyncRequest('POST', this.options.deleteDataUrl + '/ids/' + ids, {
            success: function (o) {
                this.deleteButton.set('disabled', false);
                this.options.afterRowDelete.fire();
            },
            failure: Giscuit.Util.callbackFailure,
            scope: this
        });

    }
};

Giscuit.Datatable.prototype.moveSelectedRow = function (oArgs, direction) {
    
    var selected = this.dt.getSelectedRows();
    var recordSet = this.dt.getRecordSet();
    this.currRecord = this.dt.getRecord(selected[0]);

    this.movetoRecordIndex = recordSet.getRecordIndex(this.currRecord) + direction;
    this.movetoRecord = recordSet.getRecords(this.movetoRecordIndex, 1);
    this.movetoRecord = this.movetoRecord[0];

    var first = this.currRecord.getData(this.options.keyColumn);
    var second = this.movetoRecord.getData(this.options.keyColumn);

    //Move callback
    var moveCallback = {
        success: this.moveSelectedRowSuccess,
        failure: Giscuit.Util.callbackFailure,
        scope: this
    };
    
    var request = YAHOO.util.Connect.asyncRequest('GET', this.options.moveDataUrl + "/first/" + first + "/second/" + second, moveCallback);
};

Giscuit.Datatable.prototype.moveSelectedRowSuccess = function (o) {
    try {
        var responseText = YAHOO.lang.JSON.parse(o.responseText);
    } catch(e) {
        Giscuit.Util.parseFailure(o, e);
        return;
    }
    
    if (responseText.replyCode != 200) {
        Giscuit.Util.invalidReplyCode(responseText);
        return;
    }
    
    //Unselect currect row
    this.dt.unselectAllRows();
            
    //Success
    this.dt.updateRow(this.currRecord, this.movetoRecord.getData());
    this.dt.updateRow(this.movetoRecord, this.currRecord.getData());

    //Select next
    this.dt.selectRow(this.movetoRecordIndex);
    
    this.checkInputButtons();
};

Giscuit.Datatable.prototype.checkInputButtons = function () {
     var selected = this.dt.getSelectedRows();
     var index = this.dt.getRecordIndex(selected[0]);
     
     if(selected.length != 1) {
         this.moveupButton.set('disabled', true);    
         this.movedownButton.set('disabled', true);
         return;
     }
     
     if(index == 0) {
         this.moveupButton.set('disabled', true);    
         this.movedownButton.set('disabled', false);
     } else if(index == this.dt.getRecordSet().getLength() - 1) {
         this.moveupButton.set('disabled', false);    
         this.movedownButton.set('disabled', true);
     } else {
         this.moveupButton.set('disabled', false);    
         this.movedownButton.set('disabled', false);
     }

};

//Add
Giscuit.Datatable.prototype.addRow = function () {
    this.options.onRowAdd.fire();    
};

//Refresh
Giscuit.Datatable.prototype.refresh = function() {
    this.dt.getDataSource().sendRequest('', { 
        success: this.dt.onDataReturnInitializeTable,
        failure: Giscuit.Util.callbackFailure,
        scope: this,
        argument: this.dt.getState()
    });
};

//Edit
Giscuit.Datatable.prototype.editRow = function (oArgs) {

    if(oArgs.editor != undefined) {
        var id = oArgs.editor.getRecord().getData(this.options.keyColumn);
        var column = oArgs.editor.getColumn().key;
        var data = oArgs.newData;
    } else {
        var id = oArgs.id;
        var column = oArgs.column;
        var data = oArgs.data;
    }
    
    if(data instanceof Date) {
        data = YAHOO.util.Date.format(data, {
            format: gt.config.yuiDateFormat
        });
    }
    
    var params = 'data=' + data + '&id=' + id + '&column=' + column;
    var url = this.options.updateDataUrl;

    YAHOO.util.Connect.asyncRequest('POST', url, {
        success: function (o) {
            try {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);
            } catch(e) {
                Giscuit.Util.parseFailure(o, e);
                return;
            }
            
            if (responseText.replyCode != 200) {
                Giscuit.Util.invalidReplyCode(responseText);
                return;
            }
        },
        failure: Giscuit.Util.callbackFailure,
        scope: this
    }, params);
};

//Move
YAHOO.widget.TextareaCellEditor.prototype.move = function() {
    this.textarea.style.width = "25em";
    this.textarea.style.height = "7em";
    YAHOO.widget.TextareaCellEditor.superclass.move.call(this);
};

Giscuit.Datatable.prototype.formatBooleanField = function (elCell, oRecord, oColumn, oData) {
    var value = oRecord.getData(oColumn.key);
    
    if(value === true || value == 't') {
        elCell.innerHTML = gt.translate._('yes');
    } else {
        elCell.innerHTML = gt.translate._('no');
    }
};

Giscuit.Datatable.prototype.dataChanged = function (oArgs) {
    this.dataUpdated = true;
};

Giscuit.Datatable.prototype.orderCellMousedownEvent = function (ev) {
    var Dom = YAHOO.util.Dom,
        Event = YAHOO.util.Event,
        DDM = YAHOO.util.DragDropMgr,
        lastEl = null;
        currDt = this,
        allowDragging = false;

        
    //Scroll
    var scrollTimer, dtTopY, dtBottomY,
        isScrollingUp = false,
        isScrollingDown = false;
    
    var scroll = function(coef) {
        currDt.getBdContainerEl().scrollTop += coef;
    };
    
    //Drag style
    var setStyle = function(el, currIndex, lastIndex) {
        for (var i=0; i < el.children.length; i++) {
            if(currIndex <= lastIndex) {
                  el.children[i].style.borderBottom = '1px solid #000';
            } else {
                //FIXME: Hack for gtPrivatelayersDt
                if(lastIndex == 0 && i == 0 && currDt.get('id') == 'gtPrivatelayersDt') {
                    el.children[i].style.borderTop = '2px solid #000';    
                } else {
                    el.children[i].style.borderTop = '1px solid #000';    
                }
            }
        }
    };
    
    var unsetStyle = function(el) {
        
        var rows = currDt.getTbodyEl().rows;
        for (var i=0; i < el.children.length; i++) {
            el.children[i].style.borderBottom = 0;
            el.children[i].style.borderTop = 0;
            
            //Restore previous style
            rows[0].children[i].style.borderTop = '1px solid #7F7F7F';
            rows[rows.length - 1].children[i].style.borderBottom = '1px solid #7F7F7F';
        }
    
    };
    
    ddRow = new YAHOO.util.DDProxy(currDt.getTrEl(Event.getTarget(ev).id));
    ddRow.scroll = false;
    ddRow.handleMouseDown(ev.event);
    
    ddRow.startDrag = function () {
        var proxyEl = this.getDragEl();
        var currEl = this.getEl();
        
        dtTopY = Dom.getRegion(currDt.getTheadEl()).bottom;
        dtBottomY = Dom.getRegion(currDt.getBdContainerEl()).bottom;
        
        //
        if (currDt.orderTargets[currEl.id]) {
            currDt.orderTargets[currEl.id].unreg();
            delete currDt.orderTargets[currEl.id];
        }
        currDt.orderTargets[currEl.id] = new YAHOO.util.DDTarget(currEl.id);
        
        //
        lastEl = currEl;
        
        proxyEl.innerHTML = "<table id='gtTableProxy'><tbody>" + currEl.innerHTML + "</tbody></table>";
    };

    ddRow.endDrag = function (x, y) {
        var proxyEl = this.getDragEl();
        var currEl = this.getEl();
        
        //Scroll
        if(scrollTimer) {
            scrollTimer.cancel();
        }
        isScrollingUp = false;
        isScrollingDown = false;
        
        if(lastEl != currEl) {
            var lastIndex = lastEl.sectionRowIndex,
                lastData = currDt.getRecord(lastEl).getData(),
                currIndex = currEl.sectionRowIndex,
                currData = currDt.getRecord(currEl).getData();
            
            currDt.deleteRow(currIndex);
            currDt.orderTargets[currEl.id].unreg();
            delete currDt.orderTargets[currEl.id];
            
            currDt.addRow(currData, lastIndex);
        }
        
        //
        unsetStyle(lastEl);
        
        //Save
        currDt.orderDragEndEvent.fire();
        
        Dom.setStyle(proxyEl, "visibility", "hidden");
    };
    
    ddRow.onDragOver = function (e, id) {
        var currEl = Dom.get(id);
        
        if (currEl && currEl.nodeName.toLowerCase() === "tr" && lastEl != currEl) {       
            //
            unsetStyle(lastEl);
            if(this.getEl() != currEl) {
                setStyle(currEl, this.getEl().sectionRowIndex, currEl.sectionRowIndex);
            }
            
            lastEl = currEl;
            
            DDM.refreshCache();
        }
    };
    
    ddRow.onDrag = function (e, id) {
        var y = Event.getPageY(e);
        
        //Up
        if(y < dtTopY) {
            if(isScrollingUp == false) {
                scrollTimer = YAHOO.lang.later(100, this, scroll, [-20], true);
                isScrollingUp = true;
            }
        } else {
            if(isScrollingUp == true) {
                scrollTimer.cancel();
                isScrollingUp = false;
            }
        }
            
        //Down
        if(y > dtBottomY) {
            if(isScrollingDown == false) {
                scrollTimer = YAHOO.lang.later(100, this, scroll, [20], true);
                isScrollingDown = true;
            }
        } else {
            if(isScrollingDown == true) {
                scrollTimer.cancel();
                isScrollingDown = false;
            }
        }
    };
    
};

Giscuit.Datatable.prototype.orderRowAddEvent = function (e) {
    var id = e.record.getId();
    this.orderTargets[id] = new YAHOO.util.DDTarget(id); 
}; 

Giscuit.Datatable.prototype.orderInitEvent = function () {
    var i, id, allRows = this.getTbodyEl().rows;

    for (i = 0; i < allRows.length; i++) {
        id = allRows[i].id;
        
        if (this.orderTargets[id]) {
            this.orderTargets[id].unreg();
            delete this.orderTargets[id];
        }
        
        this.orderTargets[id] = new YAHOO.util.DDTarget(id);
    }
};

Giscuit.Datatable.prototype.orderEnd = function () {
    var orderArr = [];
    
    this.dt.disable();
    
    var records = this.dt.getRecordSet().getRecords();
    for(a=0; a < records.length; a++) {
        
        //FIXME: Id is hardcoded
        var id = records[a].getData('id');
        orderArr.push(id);
    }
    
    orderArr = orderArr.join(',');
    
    var orderCallback = {
        success: function(o) {
            this.dt.undisable();
            
            try {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);
            } catch(e) {
                Giscuit.Util.parseFailure(o, e);
                return;
            }
            
            if (responseText.replyCode != 200) {
                Giscuit.Util.invalidReplyCode(responseText);
                return;
            }
        },
        scope: this,
        timeout: Giscuit.Control.callbackTimeout,
        failure: Giscuit.Control.callbackFailure
    };
    
    var request = YAHOO.util.Connect.asyncRequest('GET', this.options.orderUrl+ "/ids/" + orderArr, orderCallback);
};

YAHOO.widget.DataTable.Formatter.formatStatusField = function (elLiner, oRecord, oColumn, oData) {
    var columns = this.getColumnSet().keys;

    for(var i=0; i < columns.length; i++) {
        var record = oRecord.getData(columns[i].key);
        if (record != undefined && record != null && typeof(record) == 'object') {
            oRecord.setData(columns[i].key, "<span style='color: #FF9900;' title='" + record[0] + "'>" + record[1] + "</span>");
        }
    }

    if (oRecord.getData("hstatus") == 'U') {
        elLiner.parentNode.style.backgroundColor = '#FFFFCC';
        elLiner.innerHTML = gt.translate._('update');
    } else if (oRecord.getData("hstatus") == 'D') {
        elLiner.parentNode.style.backgroundColor = '#FF66CC';
        elLiner.innerHTML = gt.translate._('delete');
    } else if (oRecord.getData("hstatus") == 'C') {
        elLiner.parentNode.style.backgroundColor = '#CCFFCC';
        elLiner.innerHTML = gt.translate._('create');
    } else if (oRecord.getData("hstatus") == 'I') {
        elLiner.parentNode.style.backgroundColor = '#DB70DB';
        elLiner.innerHTML = gt.translate._('invalid');
    }
};