/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Map.js 3491 2013-07-17 06:00:03Z vlad.bruma $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Map
 * 
 *  Inherits from:
 *   - <OpenLayers.Map>
 */
Giscuit.Map = OpenLayers.Class(OpenLayers.Map, {
    
    pointRadius: 6,
    
    snapTolerance: 10,
    
    selectTolerance: 6,
    
    globalSelectControl: null,
    
    editgeomPath: '',
    
    lastInfoClick: null,
    
    gtEventListeners: [],
    
    panButton: null,
    
    refresh: function() {
        for (var obj in gt.layer) {
            if(typeof(gt.layer[obj].mergeNewParams) == 'undefined') {
                continue;
            }
            
            gt.layer[obj].mergeNewParams( {
                'rand' : Math.random()
            });
        }
    },

    deactivateAll: function() {
        for (var obj in gt.control) {
            if(gt.control[obj] && typeof gt.control[obj].deactivate == 'function') {
                gt.control[obj].deactivate();
            }
        }
    },
    
    removeAll: function() {
        for(var obj in gt.layer) {
            try {
                gt.map.removeLayer(gt.layer[obj]);
                gt.layer[obj].destroy();
            } catch(e) {
                debug.warn(e.message);
            }
        }
        gt.layer = {};
    },

    initialize: function(div) {
        //
        OpenLayers.IMAGE_RELOAD_ATTEMPTS = 2;
        OpenLayers.ImgPath = gt.config.httpImagePath + "/";
        OpenLayers.Projection.addTransform("EPSG:4326", "EPSG:3857", OpenLayers.Layer.SphericalMercator.projectForward);
        OpenLayers.Projection.addTransform("EPSG:3857", "EPSG:4326", OpenLayers.Layer.SphericalMercator.projectInverse);

        OpenLayers.Map.prototype.initialize.apply(this, [ div, YAHOO.lang.merge(this.getOptions()) ]);
         
        //
        for(var i=0; i < this.gtEventListeners.length; i++) {
            this.events.on(this.gtEventListeners[i]);
        }
        
        //Tooltip
        debug.info('Initializing tooltips');
        gt.tooltip = new YAHOO.widget.Tooltip("gtTooltip", {
            width: '300px'
        });
        gt.tooltip.showTooltip = function() {
            this.element.style.display = 'block';
            
            var mapRegion = YAHOO.util.Dom.getRegion('gtMap');
            var mapControls = YAHOO.util.Dom.getRegion('gtMapControls');
            var region = YAHOO.util.Dom.getRegion(this.id);
            /*
            var offsetX = 5;
            
            if(mapControls.width + region.width > mapRegion.width) {
                var offsetY = 30;
            } else {
                var offsetY = 5;
            }
            
            mapRegion[0] += mapRegion.width - region.width - offsetX;
            mapRegion[1] += offsetY;
            */
            var offsetX = 50;
            var offsetY = 55;
            if (gt.control.editgeom && gt.control.editgeom.active && 
                gt.control.layers.getEditLayer()) {
                offsetY += 12;
            }
            
            mapRegion[0] += offsetX;
            mapRegion[1] += offsetY;
            YAHOO.util.Dom.setXY(this.id, mapRegion);
            try {
                this.doShow();
            } catch (e) {
                this.doShow({}, {});
            }
            
        };
        
        gt.tooltip.hide = function() {
            gt.tooltip.element.style.display = 'none';
            YAHOO.widget.Tooltip.prototype.hide.apply(this, arguments);
        };
        
        //Permanent link (hash)
        this.events.register("moveend", this, function() {
            var hash = Giscuit.Util.getHash();
            var center = this.getCenter();
                        
            hash.lat = center.lat.toFixed(6);
            hash.lon = center.lon.toFixed(6);
            hash.zoom =  this.getZoom();
            
            Giscuit.Util.setHash(hash);
        });
        
        //Add attributions div
        var attrDiv = document.createElement('div');
        attrDiv.id = 'gtAttribution';
        attrDiv.style.zIndex = this.Z_INDEX_BASE.Control;
        attrDiv.className = 'GiscuitControlAttribution';
        this.viewPortDiv.appendChild(attrDiv);

        //
        this.globalSelectControl = new OpenLayers.Control.SelectFeature([]);
        
        //HACK:
        var globalSelectDummyLayer = new OpenLayers.Layer.Vector("globalSelectDummy");
        this.addLayer(globalSelectDummyLayer);
        this.globalSelectControl.setLayer(
            Giscuit.Util.unique(this.globalSelectControl.layers.concat([globalSelectDummyLayer]))
        );
        globalSelectDummyLayer.setVisibility(false);
        
        //
        var zoombar = YAHOO.util.Dom.getElementsByClassName('olControlPanZoomBar')[0];
        if(zoombar) {
            zoombar.title = gt.translate._('useMouseWheelToZoomInAndOut');
        }
        
        //
        this.editgeomPath = gt.config.httpPath + "/editgeom/";
    },
    
    addDefaultControls: function() {
        //
        this.addControl(this.globalSelectControl);
        this.globalSelectControl.activate();
        
        /*
        //Loading panel
        if(gt.config.guest == 'f') {
            gt.control.loading = new Giscuit.Control.LoadingPanel();
            gt.map.addControl(gt.control.loading);
        }
        */
        
        //Scale
        this.addControl(new Giscuit.Control.Scale());
        
        //ScaleLine
        if(gt.config.print == 't') {
            this.addControl(new OpenLayers.Control.ScaleLine());
        }
        
        //Pan
        this.panButton = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(14), 
            id: "gtControlPan", 
            onclick: {fn: function() {
                gt.map.deactivateAll();
            }},
            title: gt.translate._('move'),
            container: "gtMapControls"
        });
        
        //Hash
        var hash = Giscuit.Util.getHash();
        if(hash.ac) {
           gt.map.events.register("draw", this, function() {
               if(gt.control[hash.ac]) {
                   gt.control[hash.ac].activate();
               }
           });
        }
    },

    addVectorLayer: function() {
        debug.info('Initializing vector layer');
        
        var defaultStyle = OpenLayers.Util.applyDefaults({
            fillColor: "#66cccc",
            fillOpacity: 0.5,
            strokeColor: "#66cccc",
            strokeOpacity: 0.5,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["default"]);

        var style = new OpenLayers.StyleMap({
            "default" : defaultStyle
        });

        gt.layer.vector = new OpenLayers.Layer.Vector("vector", {
            styleMap : style
        });
        this.addLayer(gt.layer.vector);
    },

    addSnapLayer: function(targets, addLayer) {
        debug.info('Initializing snap');
        
        if(addLayer) {
            var defaultStyle = OpenLayers.Util.applyDefaults( {
                pointRadius: 2,
                fillColor: '#ffffff',
                fillOpacity: 1,
                strokeColor: '#ff9933',
                strokeWidth: 1
            }, OpenLayers.Feature.Vector.style['default']);
            var style = new OpenLayers.StyleMap( {
                'default': defaultStyle
            });
    
            gt.layer.snap = new OpenLayers.Layer.WFS('snap', gt.config.httpPath + '/snap/', {
                typename: 'giscuit_snap'
            }, {
                visibility: false ,
                ratio: 1,
                styleMap: style
            });
            gt.layer.snap.getFullRequestString = this.addWidthAndHeightToWfs;
            
            this.addLayer(gt.layer.snap);
    
            targets.push(gt.layer.snap);
        }
        
        gt.snap.vector = new OpenLayers.Control.Snapping( {
            layer: gt.layer.vector,
            'targets': targets,
            greedy: false ,
            defaults: {
                tolerance: this.snapTolerance
            }
        });
        
        gt.snap.edit = new OpenLayers.Control.Snapping({
            layer: gt.layer.edit,
            'targets': targets,
            greedy: false,
            defaults: {
                tolerance: this.snapTolerance
            }
        });
    },
    
    activateSnap: function() {
        for(var key in gt.snap) {
            if(typeof(gt.snap[key].activate) == 'function') {
                gt.snap[key].activate();
            }
        }
    },
    
    deactivateSnap: function() {
        for(var key in gt.snap) {
            if(typeof(gt.snap[key].activate) == 'function') {
                gt.snap[key].deactivate();
            }
        }
    },

    addEditLayer: function() {
        debug.info('Initializing edit layer');
        
        var defaultStyle = OpenLayers.Util.applyDefaults({
            pointRadius: this.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["default"]);

        var selectStyle = OpenLayers.Util.applyDefaults({
            pointRadius: this.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["select"]);

        var vertexStyle = OpenLayers.Util.applyDefaults({
            pointRadius: this.pointRadius ,
            strokeWidth: 3,
            display: 'block'
        }, OpenLayers.Feature.Vector.style['default']);
        
        var style = new OpenLayers.StyleMap( {
            "default": defaultStyle,
            "select": selectStyle,
            "vertex": vertexStyle
        });
        
        var ratio = 2;
        if(Giscuit.Util.mobileOs == 'opera') {
            ratio = 1.5;
        }
        gt.layer.edit = new OpenLayers.Layer.WFS("edit", this.editgeomPath, {
            typename: "giscuit_edit"
        }, {
            visibility: false ,
            ratio: ratio ,
            extractAttributes: "true",
            styleMap: style
        });
        gt.layer.edit.getFullRequestString = this.addWidthAndHeightToWfs;
        
        //TODO:
        if(gt.layer.edit.renderer instanceof OpenLayers.Renderer.Canvas) {
            gt.layer.edit.renderer.getFeatureIdFromEvent = 
                Giscuit.Util.bind(this.fixCanvasRatio, gt.layer.edit.renderer);
        }
        
        this.addLayer(gt.layer.edit);
    },
    
    

    getOptions: function() {
        //TileManager
        var tileManager = null;
        if(gt.config.guest == 't') {
            tileManager = new OpenLayers.TileManager();
        }
        
        //Enable zoom only for IE and Firefox
        zoomMethod = null;
        if(YAHOO.env.ua.gecko || YAHOO.env.ua.ie) {
            var zoomMethod = OpenLayers.Map.prototype.zoomMethod;
        }
        
        //Controls
        var controls = [ 
            new OpenLayers.Control.Navigation({
                /**
                mouseWheelOptions: {
                    interval: 20, 
                    cumulative: false
                },
                */
                dragPanOptions: {
                    enableKinetic: false
                }
            })
        ];
        if(gt.config.link == 'f' && gt.config.print == 'f') {
            controls.push(
                new OpenLayers.Control.PanZoomBar({
                    position: new OpenLayers.Pixel(8, 28),
                    panIcons: false
                })
            );
        }
        
        return {
            zoomMethod: zoomMethod,
            theme: null ,
            maxExtent: new OpenLayers.Bounds(gt.config.minx, gt.config.miny, gt.config.maxx, gt.config.maxy),
            numZoomLevels: gt.config.resolutions.length ,
            allOverlays: true,
            units: gt.config.units,
            controls: controls,
            resolutions: gt.config.resolutions,
            projection: new OpenLayers.Projection("EPSG:" + gt.config.srid),
            fallThrough: true,
            tileManager: tileManager
        };
    },

    setBounds: function() {
        var hash = Giscuit.Util.getHash();

        if (hash.lat != undefined && hash.lon != undefined && hash.zoom != undefined) {
            var pos = new OpenLayers.LonLat(parseFloat(hash.lon), parseFloat(hash.lat));
            this.setCenter(pos, parseInt(hash.zoom));
            //FIXME:
            this.setCenter(pos, parseInt(hash.zoom));
        } else {
            this.zoomToExtent(this.maxExtent);
            //FIXME:
            this.zoomToExtent(this.maxExtent);
        }
    },

    getLayersString: function(visibile) {
        if(gt.config.guest == 'f') {
            return '';
        }
        
        if(visibile == undefined) {
            visibile = true;
        }
        
        var data = [];
        for(var a=0; a < this.layers.length; a++) {
            var layer = this.layers[a];

            if((visibile == true && layer.visibility == false) 
                || layer.CLASS_NAME == "OpenLayers.Layer.Vector") {
                
                continue;
            }

            for(var b in gt.control.layers.records) {
                var record = gt.control.layers.records[b];
                
                if(record.layerClass == layer.name) {
                    if(record.classes instanceof Array) {
                        data = data.concat(record.classes);
                    } else {
                        data = data.concat([record.classes]);
                    }
                    break;
                }
            }

        }
        
        return '/layers/' + Giscuit.Util.unique(data).join(',');
    },
    
    addWidthAndHeightToWfs: function(newParams, altUrl) {
        //Add map size
        var imageSize = this.map.getSize();
        this.params.WIDTH = imageSize.w * this.ratio;
        this.params.HEIGHT = imageSize.h * this.ratio;
        
        var projectionCode = this.projection.getCode() || this.map.getProjection();
        this.params.SRS = (projectionCode == "none") ? null : projectionCode;
        
        return OpenLayers.Layer.Grid.prototype.getFullRequestString.apply(this, arguments);
    },
    
    getControlIcon: function(value, title) {
        if(title != undefined) {
            title = 'title="' + title + '"';
        } else {
            title = '';
        }
        
        if(isNaN(value)) {
            var html = "<em class='gtControlIcon' " + title + " + style='background: url(" + gt.config.httpImagePath + "/" + value + ") " + "0px center no-repeat;'>&nbsp;</em>";
        } else {
            var html = "<em class='gtControlIcon' " + title + " + style='background: url(" + gt.config.httpImagePath + "/sprite.png) " + value * -16  + "px center no-repeat;'>&nbsp;</em>";
        }
        
        return html;
        
    },
    
    getStyleRules: function(data, forEdit) {
        if(typeof(forEdit) == 'undefined') {
            forEdit = false;
        }
        
        var sldFormat = new OpenLayers.Format.SLD();
        var sldStyle = sldFormat.read(data);
        
        for (className in sldStyle.namedLayers) break;
        var rules = sldStyle.namedLayers[className].userStyles[0].rules.reverse();
       
        //Parse rules
        for(var i=0; i < rules.length; i++) {
            for (var symbolizerType in rules[i].symbolizer) break;
            var symbolizer = rules[i].symbolizer[symbolizerType];
            
            //
            if(symbolizerType == 'Line' && symbolizer.strokeWidth < 3) {
                symbolizer.strokeWidth = 3;
            }

            //
            if(typeof(symbolizer.fillOpacity) == 'undefined') {
                if(! forEdit) {
                    symbolizer.fillOpacity = 1;
                } else {
                    symbolizer.fillOpacity = 0.75;
                }
            }
            
            //During editing polygons with no background color can now be selected if you click inside of them 
            if(symbolizerType == 'Polygon' && typeof(symbolizer.fillColor) == 'undefined') {
                symbolizer.fill = true;
                symbolizer.fillOpacity = 0;
            }
            
            //FIXME: maxScaleDenominator
            if(typeof(rules[i].maxScaleDenominator) != 'undefined') {
                delete rules[i].maxScaleDenominator;
            }
            
            //FIXME: maxScaleDenominator
            if(typeof(rules[i].minScaleDenominator) != 'undefined') {
                delete rules[i].minScaleDenominator;
            }
            
            //Delete 
            delete rules[i].symbolizer.Text;
            
            //Label
        }
        
        return rules;
    },
    
    addJsInfoResults: function(data) {
        var tolerance = gt.map.selectTolerance;

        var xy = gt.map.getPixelFromLonLat(gt.map.lastInfoClick);
        
        //Bounds
        var topLeft = gt.map.getLonLatFromViewPortPx({
            'x': xy.x - tolerance,
            'y': xy.y - tolerance
        });
        var bottomRight = gt.map.getLonLatFromViewPortPx({
            'x': xy.x + tolerance,
            'y': xy.y + tolerance
        });
        
        var windowBounds = new OpenLayers.Bounds();
        windowBounds.extend(topLeft);
        windowBounds.extend(bottomRight);
        var windowBoundsGeometry = windowBounds.toGeometry();
        
        //
        for(var a=0; a < gt.control.layers.records.length; a++) {
            var layerSettings = gt.control.layers.records[a];
            
            if(gt.config.guest == 't') {
                if(! (layerSettings.jsLayer || layerSettings.js_layer)) {
                    continue;
                }
                
                //FIXME:
                layerSettings['class'] = layerSettings['layerClass'];
            } else {
                if(! layerSettings.js_layer ||
                    ! layerSettings.view ||
                    ! layerSettings.select) {
                    
                    continue;
                }
            }
            
            var layer = gt.layer[layerSettings['class']];
            if(typeof(layer.features) == 'undefined') {
                continue;
            }
            
            var match = [];
            
            for(var b=0; b < layer.features.length; b++) {
                var feature = layer.features[b];
                                
                //
                if(match.length > 5 || 
                    ! feature.getVisibility() ||
                    ! (windowBounds.intersectsBounds(feature.geometry.bounds) && windowBoundsGeometry.intersects(feature.geometry))) {
                    
                    continue;
                }
                
                match.push(feature);
            }
            
            if(match.length > 0) {
                var layerResult = {
                    'expanded': true,
                    'type': 'HTML',
                    'html': layerSettings.name,
                    'children': []
                }
                
                //Get default field
                if(typeof(layer.defaultField) != 'undefined') {
                    var defaultField = layer.defaultField;
                } else {
                    for(var defaultField in match[0].attributes) {
                        break;
                    }
                }
                
                for(var c=0; c < match.length; c++) {
                    var featureResult = {
                        'type': 'HTML',    
                        'html': match[c].attributes[defaultField],
                        'the_geom': match[c].geometry.toString(),
                        'children': []
                    }
                    
                    for(var d in match[c].attributes) {
                        var fieldResult = {
                            'type': 'HTML',
                            'hasIcon': false,
                            'html': gt.translate._(d) + ': ' + match[c].attributes[d] 
                        }
                        
                        featureResult.children.push(fieldResult);
                    }
                    
                    layerResult.children.push(featureResult);
                }
                
                data.push(layerResult);
            }
        }

    },
    
    //TODO:
    fixCanvasRatio: function(evt) {
        var featureId, feature;
        
        if (this.hitDetection && this.root.style.display !== "none") {
            // this dragging check should go in the feature handler
            if (!this.map.dragging) {
                var ratio = 2;
                var xy = evt.xy;
                
                if(ratio == 1) {
                    var x = xy.x | 0;
                    var y = xy.y | 0;
                } else {
                    var x = xy.x + Math.round(this.size.w / ratio / 2) | 0;
                    var y = xy.y + Math.round(this.size.h / ratio / 2) | 0;
                }

                var data = this.hitContext.getImageData(x, y, 1, 1).data;
                if (data[3] === 255) { // antialiased
                    var id = data[2] + (256 * (data[1] + (256 * data[0])));
                    if (id) {
                        featureId = "OpenLayers_Feature_Vector_" + (id - 1 + this.hitOverflow);
                        featureId2 = "OpenLayers.Feature.Vector_" + (id - 1 + this.hitOverflow);
                        
                        if(this.features[featureId]) {
                            feature = this.features[featureId][0];
                        } else if (this.features[featureId2]) {
                            feature = this.features[featureId2][0];
                        }
                        
                    }
                    
                }
            }
        }
        return feature;
    },
    
    orderRecordsDesc: function(records) {
        //Order the array by order column
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;
            
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        
        return records.sort(byOrder).reverse();
    },
    
    orderRecordsAsc: function(records) {
        //Order the array by order column
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;
            
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        
        return records.sort(byOrder);
    },
    
    CLASS_NAME : "Giscuit.Map"
});