/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Geolocation.js 1749 2011-09-15 11:10:45Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.geolocation = new Giscuit.Mobile.Control.Geolocation();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Geolocation
 * 
 *  Inherits from:
 *   - <Giscuit.Control.Geolocation>
 */
Giscuit.Mobile.Control.Geolocation = OpenLayers.Class(Giscuit.Control.Geolocation, {
    
    heading: null,
    
    clickControl: null,
    
    updateOrientationHandler: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
          
        //Static
        this.clickControl = new Giscuit.Control.Click({
            'clickCallback': Giscuit.Util.bind(this.staticGeolocation, this)
        });
        gt.map.addControl(this.clickControl);
        
        //Geolocation
        if(typeof(navigator.geolocation) != 'undefined') {
            this.gl = navigator.geolocation;
        }
        
        //
        if(this.gl != null) {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtGeolocationButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -64px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('location') + '</div>'
                + '</div></div>');
            
            var taphold = false;
            
            $("#gtGeolocationButton").on(gt.config.clickEvt, this, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                if(taphold) {
                    taphold = false;
                    return false;
                }
                //
                evt.data.clickControl.deactivate();
                evt.data.manually = false;
                
                //
                if($(this).hasClass('ui-btn-active')) {
                    evt.data.stop();
                    $(this).removeClass('ui-btn-active');
                } else {
                    evt.data.start();
                    $(this).addClass('ui-btn-active');
                }
                
                taphold = false;
                return false;
            });
            
            $("#gtGeolocationButton").on('taphold', this, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                //
                evt.data.clickControl.deactivate();
                evt.data.manually = false;
                evt.data.stop();
                
                //
                $("#gtGeolocationButton .gtMapControlsElText").html(gt.translate._('manually'));
                $(this).addClass('ui-btn-active');

                //
                evt.data.clickControl.activate();
                                
                taphold = true;
                return false;
            });
        }
        
        //GpsGate
        if(! Giscuit.Util.isMobile) {
            GpsGate.Client.getVersion(Giscuit.Util.bind(function(evt) {
                this.gl = 'gpsgate';
            }, this));
        }
    },
    
    startOrientation: function() {
        if ('ondeviceorientation' in window) {
            this.updateOrientationHandler = Giscuit.Util.bind(this.updateOrientation, this);
            window.addEventListener('deviceorientation', this.updateOrientationHandler, true);
        } else if(navigator.compass) {
            this.updateOrientationHandler = navigator.compass.watchHeading(
                Giscuit.Util.bind(this.updateOrientation, this),
                null,
                {}
            );
        }
    },
    
    stopOrientation: function() {
        if ('ondeviceorientation' in window) {
            window.removeEventListener('deviceorientation', this.updateOrientationHandler, true);
        } else if(navigator.compass) {
            navigator.compass.clearWatch(this.updateOrientationHandler);
        }
        
        this.updateOrientationHandler = null;
        this.heading = null;
        
    },
    
    updateOrientation: function(evt) {
        //
        if(! this.point) {
            return;
        }
        
        //Style
        if(this.heading === null && 
            (evt.webkitCompassHeading || evt.magneticHeading || evt.alpha)
            ) {
            
            this.point.style.rotation = this.heading;
            this.point.style.externalGraphic = gt.config.httpImagePath + '/geolocation_heading.png';
            this.point.style.pointRadius = 15 / 2;
            
            gt.layer.vector.drawFeature(this.point);
        }
        
        //Heading
        if (typeof(evt.webkitCompassHeading) != 'undefined') {
            this.heading = evt.webkitCompassHeading;
            if (typeof(window.orientation) != 'undefined') {
                this.heading += window.orientation;
            }
        } else if (typeof(evt.magneticHeading) != 'undefined') {
            this.heading = evt.magneticHeading;
        } else {
            this.heading = 270 - evt.alpha + window.orientation;
        }
        while (this.heading >= 360) {
            this.heading -= 360;
        }
        while (this.heading < 0) {
            this.heading += 360;
        }
        
        //Update
        if(Math.abs(this.heading - this.point.style.rotation) > 1) {
            this.point.style.rotation = this.heading;
            gt.layer.vector.drawFeature(this.point);
        }
    },
    
    error: function(error) {
        $("#gtGeolocationButton").removeClass('ui-btn-active');
        
        var msg = '';
        switch(error.code) {
            case error.PERMISSION_DENIED: msg = 'geolocationDataNotShared';
            break;
        
            case error.POSITION_UNAVAILABLE: msg = 'geolocationPositionNotDetected';
            break;
            
            case error.TIMEOUT: msg = 'geolocationTimedOut';
            break;
            
            default: msg = 'geolocationUnknownError';
            break;
        }
                
        Giscuit.Util.jqmMessage(gt.translate._(msg));
        this.stop();
    },
    
    start: function () {
        $("#gtGeolocationButton").addClass('ui-btn-active');
        
        this.startOrientation();
        Giscuit.Control.Geolocation.prototype.start.apply(this, [arguments]);
    },
    
    stop: function () {
        $("#gtGeolocationButton .gtMapControlsElText").html(gt.translate._('location'));
        $("#gtGeolocationButton").removeClass('ui-btn-active');
        
        this.stopOrientation();
        Giscuit.Control.Geolocation.prototype.stop.apply(this, [arguments]);
    },
    
    updateCallback: function(position) {
        $("#gtGeolocationButton .gtMapControlsElText")
            .html((Math.round(position.coords.accuracy * 100) / 100) + 'm');
    },
    
    staticGeolocation: function(evt) {
        //
        this.clickControl.deactivate();

        //
        var point = gt.map.getLonLatFromViewPortPx(evt.xy);
        point.transform(gt.map.projection, new OpenLayers.Projection("EPSG:4326"));
        var position = {
            coords: {
                longitude: point.lon,
                latitude: point.lat,
                accuracy: 0
            }
        };
        this.update(position);
        
        //
        this.manually = true;
    },
    
    CLASS_NAME: "Giscuit.Mobile.Util.Geolocation"
});