/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Map.js 1403 2011-06-12 19:01:12Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Map
 * 
 *  Inherits from:
 *   - <Giscuit.Map>
 */
Giscuit.Mobile.Map = OpenLayers.Class(Giscuit.Map, {

    pointRadius: 15,
    
    snapTolerance: 20,
    
    selectTolerance: 15,
    
    initialize: function(div) {
        debug.info('Initializing map');
        
        OpenLayers.IMAGE_RELOAD_ATTEMPTS = 2;
        
        OpenLayers.ImgPath = gt.config.httpImagePath + "/";
        
        OpenLayers.Projection.addTransform("EPSG:4326", "EPSG:3857", OpenLayers.Layer.SphericalMercator.projectForward);
        OpenLayers.Projection.addTransform("EPSG:3857", "EPSG:4326", OpenLayers.Layer.SphericalMercator.projectInverse);
                
        OpenLayers.Map.prototype.initialize.apply(this, [div, this.getOptions()]);
                
        //
        for(var i=0; i < this.gtEventListeners.length; i++) {
            this.events.on(this.gtEventListeners[i]);
        }
        
        //Permanent link
        this.events.register("moveend", this, function() {
            localStorage.gtMapExtent = this.getExtent().toString();
        });
        
        //
        this.globalSelectControl = new OpenLayers.Control.SelectFeature([]);
        
        //FIXME:
        var globalSelectDummyLayer = new OpenLayers.Layer.Vector("globalSelectDummy");
        this.addLayer(globalSelectDummyLayer);
        this.globalSelectControl.setLayer(
            Giscuit.Util.unique(this.globalSelectControl.layers.concat([globalSelectDummyLayer]))
        );
        globalSelectDummyLayer.setVisibility(false);
        
        //
        this.editgeomPath  = gt.config.httpMobilePath + "/editgeom/";
    },
        
    setBounds: function() {
        //Url
        var pathName = window.location.pathname.split('/');
        var lon, lat, zoom, ac = null;
        for(var i=0; i < pathName.length; i++) {
            if(pathName[i] == 'lon') {
                var lon = pathName[i + 1]; 
            } else if(pathName[i] == 'lat') {
                var lat = pathName[i + 1]; 
            } else if(pathName[i] == 'zoom') {
                var zoom = pathName[i + 1]; 
            } else if(pathName[i] == 'ac') {
                var ac = pathName[i + 1]; 
            }
            
        }
        
        if(lon && lat && zoom) {
            var pos = new OpenLayers.LonLat(parseFloat(lon), parseFloat(lat));
            this.setCenter(pos, parseInt(zoom));
            //FIXME:
            this.setCenter(pos, parseInt(zoom));
        } else {
            //Local storage
            var savedExtent = localStorage.gtMapExtent;
        
            if(savedExtent) {
                var extent = OpenLayers.Bounds.fromString(savedExtent);
            } else {
                var extent = this.maxExtent;
            } 
        
            this.zoomToExtent(extent, true);
            //FIXME:
            this.zoomToExtent(extent, true);
        }
        
        if(ac) {
            gt.map.events.register("draw", this, function() {
                $(document).ready(function() {
                    if(gt.control[ac]) {
                        gt.control[ac].activate();
                    }
                });
            });
        }
    },

    addDefaultControls: function() {
        //
        this.addControl(this.globalSelectControl);
        this.globalSelectControl.activate();
        
        //Toggle controls panel
        $("#gtToggleControlsButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            if($(this).hasClass('ui-btn-active')) {
                $(this).removeClass('ui-btn-active');
                $("#gtMapControls").css('display', 'none');
                
                $(this).css('bottom', '5px');
            } else {
                $(this).addClass('ui-btn-active');
                $("#gtMapControls").css('display', 'block');
                
                $(this).css('bottom', $("#gtMapControls").height() + 'px');
            }
            
            return false;
        });
        
        //Zoom
        $("#gtMapPlus").on(gt.config.clickEvt, this, function(evt) {
            evt.data.zoomIn();
            gt.init.layout.buttonClick(this);
            
            return false;
        });
        $("#gtMapMinus").on(gt.config.clickEvt, this, function(evt) {
            evt.data.zoomOut();
            gt.init.layout.buttonClick(this);
            
            return false;
        });
        
        //Prevent default
        $("#gtMapControls, #gtMapNavigation, gtMapToggleControls").on('touchmove', function(evt) {
           return false;
        });

    },

    getOptions : function() {
        //TileManager
        var tileManager = null;
        if(gt.config.guest == 't') {
            tileManager = new OpenLayers.TileManager();
        }
        
        //Controls
        var controls = [
            new OpenLayers.Control.TouchNavigation({
                dragPanOptions: {
                    enableKinetic: true
                }
            })
        ];
        
        return {
            theme: null,
            maxExtent: new OpenLayers.Bounds(gt.config.minx, gt.config.miny, gt.config.maxx, gt.config.maxy),
            numZoomLevels: gt.config.resolutions.length ,
            allOverlays: true,
            units: gt.config.units,
            controls: controls,
            resolutions: gt.config.resolutions,
            projection: new OpenLayers.Projection("EPSG:" + gt.config.srid),
            fallThrough: false,
            tileManager: tileManager,
            zoomMethod: null
        };
        
    },
    
    showMetadata: function(layer) {
        //Create a html list of data
        var text = '';
        var title = '';
        if (layer.metadata instanceof Array) {
            text += '<ul data-role="listview">';
            for (var i = 0; i < layer.metadata.length; i++) {
                text += '<li><h3><i>' + gt.translate._(layer.metadata[i].ows_title) + ':</i></h3></li>';
                
                for (var key in layer.metadata[i]) {
                    if (layer.metadata[i].hasOwnProperty(key) && key != 'ows_title') {
                        text += '<li>' + gt.translate._(key) + ': ' + layer.metadata[i][key] + '</li>';
                    }
                }
            }
            text += '</ul>';
            title = '<h1> </h1>';
        } else {
            text += '<ul data-role="listview">';
            if (layer.maxscaledenom) {
                text += '<li>' + gt.translate._('maxscaledenom') + ': 1:' + layer.maxscaledenom + '</li>';
            }
            if (layer.minscaledenom) {
                text += '<li>' + gt.translate._('minscaledenom') + ': 1:' + layer.minscaledenom + '</li>';
            }
            
            for (var key in layer.metadata) {
                if (layer.metadata.hasOwnProperty(key) && key != 'ows_title') {
                    text += '<li>' + gt.translate._(key) + ': ' + layer.metadata[key] + '</li>';
                }
            }
            text += '</ul>';
            title = '<h1>' + gt.translate._(layer.metadata.ows_title) + '</h1>';
        }

        var html = 
        '<div data-role="page" id="gtMetadata" data-dom-cache="true" data-url="gtMetadata">' +
            '<div data-role="header">' +
                title +
            '</div>' +
            '<div data-role="content">' + 
                text +
            '</div>' +
        '</div>';
                
        $('#gtMetadata').off();
        $('#gtMetadata').remove();
        
        $.mobile.pageContainer.append(html);
        $('#gtMetadata').css('min-height', '100%');
        
        $.mobile.changePage('#gtMetadata', {'role': 'dialog', 'transition': 'none'});
    },
    
    removeReservedAttributes: function(data) {
        var newData = $.extend({}, data);
        
        //hstatus
        if(typeof(newData.hstatus) != 'undefined') {
            delete newData.hstatus;
        }
        
        //__hdate
        if(typeof(newData.__hdate) != 'undefined') {
            delete newData.__hdate;
        }
        
        //gid
        if(typeof(newData.gid) != 'undefined') {
            delete newData.gid;
        }
        
        //MAX_FILE_SIZE
        if(typeof(newData.MAX_FILE_SIZE) != 'undefined') {
            delete newData.MAX_FILE_SIZE;
        }
        
        //__layerClass
        if(typeof(newData.__layerClass) != 'undefined') {
            delete newData.__layerClass;
        }
        
        //__featureId
        if(typeof(newData.__featureId) != 'undefined') {
            delete newData.__featureId;
        }
        
        //__fileid_
        for (var field in newData) {
            if (field.substr(0, 9) == '__fileid_') {
                delete newData[field];
            }
        }
        
        return newData;
    },
    
    addBboxHandler: function(control, layer, tolerance) {
        var self = this;
        
        if (control.multipleSelect()) {
            debug.warn('Multiple selection is not supported');
            return;
        }
        
        control.handlers.click = new OpenLayers.Handler.Click(control, 
            {
                click: function(evt) {
                    control.unselectAll();
                    
                    var selectedFeature = self.getClosestFeature(layer, evt.xy, tolerance);
                    
                    if(selectedFeature) {
                        control.select(selectedFeature);
                    }
                }
            }, 
            {}
        );
        
        control.events.on({
            "activate": function() {
                control.handlers.feature.deactivate();
                control.handlers.click.setMap(control.map);
                control.handlers.click.activate();
            },
            "deactivate": function() {
                control.handlers.click.deactivate();
            }
        });
    },
    
    getClosestFeature: function(layer, xy, tolerance) {
        //
        var resultKeys = [];
        var result = [];
        var selectedFeature = null;
        
        //Bounds
        var topLeft = gt.map.getLonLatFromViewPortPx({
            'x': xy.x - tolerance,
            'y': xy.y - tolerance
        });
        var bottomRight = gt.map.getLonLatFromViewPortPx({
            'x': xy.x + tolerance,
            'y': xy.y + tolerance
        });
        
        var bounds = new OpenLayers.Bounds();
        bounds.extend(topLeft);
        bounds.extend(bottomRight);
        
        var boundsGeometry = bounds.toGeometry();
        var center = boundsGeometry.getCentroid();
        
        //Get best match
        for(var i=0, len = layer.features.length; i<len; ++i) {
            var feature = layer.features[i];
            if (! feature.getVisibility()) {
                continue;
            }
                                
            if (bounds.intersectsBounds(feature.geometry.bounds) &&
                boundsGeometry.intersects(feature.geometry)) {
                
                var distance = feature.geometry.distanceTo(center);
                resultKeys.push(distance);
                result[distance] = feature;
            }
        }
        resultKeys.sort();
        if(resultKeys.length) {
            selectedFeature = result[resultKeys[0]];
        }
        
        return selectedFeature;
    },
    
    fixGoogleLayers: function() {
        if(typeof(OpenLayers.Layer.Google) == 'undefined') {
            return;
        } 
        
        for (var obj in gt.layer) {
            if(gt.layer[obj] instanceof OpenLayers.Layer.Google) {
                google.maps.event.trigger(gt.layer[obj].mapObject, "resize");
                gt.layer[obj].redraw();
            }
        }
    },
    
    CLASS_NAME : "Giscuit.Mobile.Map"
});