<?php

class EditgeomControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    protected $_layerName = 'Worldboundaries';
    
    protected $_wfsParams = array(
        'TYPENAME' => 'giscuit_edit' , 
        'SERVICE' => 'WFS' , 
        'VERSION' => '1.0.0' , 
        'REQUEST' => 'GetFeature' , 
        'WIDTH' => '1736' , 
        'HEIGHT' => '1164' , 
        'SRS' => 'EPSG%3A4326' , 
        'BBOX' => '30.127802348379,46.421546685608,30.132458663227,46.4246687769'
    );
    
    protected $_testCountryParams = array (
        'name' => 'test_cntry',
        'iso_a2' => 'ST',
        'iso_a3' => 'TST',
        'people' => '123',
        'dial' => '666',
        'geometry' => 'POLYGON((30.150630457031 46.624943761718,30.373103601563 46.594731359374,30.260493738281 46.175877599609,29.731776697266 46.298100499999,30.150630457031 46.624943761718))'
    );
    
    protected function _getFieldFromClass ($layer, $field)
    {
        $layersTable = new Giscuit_Db_Table_Layers();
        $row = $layersTable->getItems(array(
            'whereClass' => $layer
        ));
        foreach ($row as $r) {
            $field = $r[$field];
        }
        return $field;
    }
    
    protected function _setEdit ($value)
    {
        //
        $id = $this->_getFieldFromClass($this->_layerName, 'id');
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/edit/id/' . $id . '/column/edit/value/' . $value);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        
        $this->clear();
    }
    
    protected function _getGidByName($name)
    {
        $db = Zend_Registry::get('db');
        $config = Zend_Registry::get('config');
        
        $tableName = $this->_getFieldFromClass($this->_layerName, 'table');
        
        $result = $db->fetchAll("SELECT gid FROM {$config->database->schema}_layers.{$tableName} WHERE name = '{$name}'");

        if (isset($result[0])) {
            return $result[0]->gid;
        } else {
            return false;
        }
    }
    
    public function testIndexAction ()
    {
        $this->_testLoginActionSuccess();
        $this->_setEdit('true');
        
        $queryString = '';
        foreach ($this->_wfsParams as $key => $value) {
            $queryString .= "{$key}={$value}&";
        }
        
        $url = '/' . $this->_language . '/default/editgeom/?' . $queryString;
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        $this->assertContains('<gml:coordinates>33.451840,52.333808', $response);

    }
    
    public function testGetAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/default/editgeom/get/version/0');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        $this->assertContains('gtEditattrForm', $response['form']);
        
    }
    
    public function testAddAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->request->setMethod('POST')->setPost($this->_testCountryParams);
        $this->dispatch('/' . $this->_language . '/default/editgeom/add/');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        $this->assertTrue($response['success']);
        
    }
    
    public function testEditAction()
    {
        $this->_testLoginActionSuccess();
        
        $countryGid = $this->_getGidByName('test_cntry');
        $this->_testCountryParams['name'] = 'test_cntry_edited';
        $this->_testCountryParams['geometry'] = 'POLYGON((30.15063 46.624944,30.373104 46.594731,30.520046070312503 46.333119441406254,30.260494 46.175878,29.731777 46.2981,30.15063 46.624944))';
        $this->request->setMethod('POST')->setPost($this->_testCountryParams);
        
        $this->dispatch('/' . $this->_language . '/default/editgeom/edit/id/' . $countryGid);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        $this->assertTrue($response['success']);

        $editedCountryGid = $this->_getGidByName('test_cntry_edited');
        
        $this->assertEquals($countryGid, $editedCountryGid);
    }
    
    public function testDeleteAction()
    {
        $this->_testLoginActionSuccess();
        
        $editedCountryGid = $this->_getGidByName('test_cntry_edited');
        
        $this->dispatch('/' . $this->_language . '/default/editgeom/delete/id/' . $editedCountryGid);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        
        $inexistentGid = $this->_getGidByName('test_cntry_edited');
        
        $this->assertFalse($inexistentGid);
    }

    public function unsetEditable()
    {
        $this->_testLoginActionSuccess();
        
        $this->_setEdit('false');
    }
}