<?php

class RastercutControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    
    protected $_indexParams = array (
        'comment' => 'test comment',
        'geometry' => 'POLYGON((26.58524227142 48.355628967307,27.788245201108 48.630287170432,29.09012508392 48.196327209494,30.271155357358 46.295692443869,28.403479576108 45.416786193869,27.96951961517 45.389320373557,28.128821372983 46.652748107932,26.49735164642 48.278724670432,26.58524227142 48.355628967307))',
        'resolution' => 200,
        'scale' => 1730249,
        'tiled' => 0
    );
    
    public function testGetFormAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/tool/rastercut/getform');
        
        $this->assertModule('tool');
        $this->assertController('rastercut');
        $this->assertAction('getform');
        $this->assertQuery('form#gtRastercutForm');
    }
    
    
    public function testIndexAction()
    {
        $session = new Zend_Session_Namespace('RastercutControllerTest');
        
        $this->_testLoginActionSuccess();
        
        //Delete all previous records 
        $rastercutTable = new Giscuit_Db_Table_Layer_Rastercut(array(
            'name' => 'rastercut'
        ));
        $deleteWhere = $rastercutTable->getAdapter()->quoteInto('comment = ?', $this->_indexParams['comment']);
        $rastercutTable->delete($deleteWhere);

        //Dispatch
        $this->request->setMethod('POST')->setPost($this->_indexParams);
        $this->dispatch('/' . $this->_language . '/tool/rastercut');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $filePath = GT_TMP_PATH . DIR_SEP . $response['file'] . '.zip';
        
        $this->assertEquals($response['replyCode'], 200);
        $this->assertTrue($response['valid']);
        $this->assertFileExists($filePath);
        
        //
        $zip = new ZipArchive();
        $res = $zip->open($filePath);
        if ($res) {
            $zip->extractTo(GT_TMP_PATH);
            $size = getimagesize(GT_TMP_PATH . DIR_SEP .'map.jpg');
            $wdl = file_get_contents(GT_TMP_PATH . DIR_SEP .'map.jgw');
        }
        $zip->close();

        //
        $this->assertFileExists(GT_TMP_PATH . DIR_SEP .'map.jpg');
        $this->assertEquals($size[0], 1908);
        $this->assertEquals($size[1], 1638);
        unlink(GT_TMP_PATH . DIR_SEP .'map.jpg');
        
        //
        $this->assertFileExists(GT_TMP_PATH . DIR_SEP .'map.jgw');
        $this->assertContains('26.49833946206', $wdl);
        $this->assertContains('48.62753986827', $wdl);
        unlink(GT_TMP_PATH . DIR_SEP .'map.jgw');
        
    }
    
    public function testManageAction()
    {
        $this->_testLoginActionSuccess();
                    
        $this->dispatch('/' . $this->_language . '/tool/rastercut/manage');
        
        $this->assertModule('tool');
        $this->assertController('rastercut');
        $this->assertAction('manage');
        $this->assertQuery('form#gtRastercutManageForm');
    }
    
    public function testManagedatatableAction()
    {
        $dateBegin = new Zend_Date(0);
        $dateEnd = new Zend_Date();        
        
        $this->_testLoginActionSuccess();
        
        $this->request->setMethod('POST')->setPost(array(
            'dateBegin' => $dateBegin->get(GT_DATETIME_STYLE_PHP) ,
            'dateEnd' => $dateEnd->get(GT_DATETIME_STYLE_PHP) ,
        
        ));
        $this->dispatch('/' . $this->_language . '/tool/rastercut/managedatatable');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        
        $recordFound = false;
        foreach ($response['records'] as $record) {
            if($record['comment'] == $this->_indexParams['comment']) {
                $recordFound = true;
                break;
            }
        }
        $this->assertTrue($recordFound);
    }
}

?>